static char * readme_env[] = {
   "Unix Environment Variables Used by AFNI\n" ,
   "=======================================\n" ,
   "The AFNI program allows you to use several Unix environment variables\n" ,
   "to influence its behavior.  The mechanics of setting an environment\n" ,
   "variable depend on which shell you are using.  To set an environment\n" ,
   "variable named \"FRED\" to the string \"Elvis\":\n" ,
   "\n" ,
   "   csh or tcsh:  setenv FRED Elvis\n" ,
   "   bash or ksh:  FRED=Elvis ; export FRED\n" ,
   "\n" ,
   "Normally, these commands would go in your .cshrc or .profile files,\n" ,
   "so that they would be invoked when you login.  If in doubt, consult\n" ,
   "your local Unix guru.  If you don't have one, well....\n" ,
   "\n" ,
   "You don't NEED to set any of these variables -- AFNI will still work\n" ,
   "correctly.  But they are an easy way to set up certain defaults to\n" ,
   "make AFNI a little easier on your neocortex and hippocampus.\n" ,
   "\n" ,
   "N.B.: Changes to environment variables AFTER you start a program will\n" ,
   "      not be seen by that program, since each running program gets\n" ,
   "      a private copy of the entire set of environment variables when\n" ,
   "      it starts.  This is a standard Unix feature, and is not specific\n" ,
   "      to AFNI.  Some variables can be set internally in AFNI using\n" ,
   "      the \"Edit Environment\" control from the \"Datamode->Misc\" menu\n" ,
   "      or from the image window Button-3 popup menu.  Such variables\n" ,
   "      are marked with \"(editable)\" in the descriptions below.\n" ,
   "\n" ,
   "N.B.: Some variables below are described as being of \"YES/NO\" type.\n" ,
   "      This means that they should either be set to the value \"YES\"\n" ,
   "      or to the value \"NO\".\n" ,
   "\n" ,
   "N.B.: You can now set environment variables on the 'afni' command\n" ,
   "      line; for example:\n" ,
   "        afni -DAFNI_EDGIZE_OVERLAY=YES -DAFNI_SESSTRAIL=3\n" ,
   "      This may be useful for a 'one time' situation, or as an alias.\n" ,
   "      -- RWCox - 22 Mar 2005\n" ,
   "\n" ,
   "N.B.: At the end of this file is a list of several environment variables\n" ,
   "      that affect the program 3dDeconvolve, rather than the interactive\n" ,
   "      AFNI program itself.\n" ,
   "\n" ,
   "********************************************************\n" ,
   "June 1999: Setting environment variables in file .afnirc\n" ,
   "********************************************************\n" ,
   "You can now set environment variables for an interactive AFNI run in the\n" ,
   "setup (.afnirc) file.  This is provided as a convenience.  An example:\n" ,
   "\n" ,
   "***ENVIRONMENT\n" ,
   "  AFNI_HINTS = YES\n" ,
   "  AFNI_SESSTRAIL = 3\n" ,
   "\n" ,
   "Note that the spaces around the \"=\" sign are required.  See README.setup\n" ,
   "for more information about the possible contents of .afnirc besides the\n" ,
   "environment variables.\n" ,
   "\n" ,
   "A few other programs in the AFNI package also read the ***ENVIRONMENT\n" ,
   "section of the .afnirc file.  This is needed so that environment settings\n" ,
   "that affect those programs (e.g., AFNI_COMPRESSOR for auto-compression of\n" ,
   "output datasets) can be properly initialized in .afnirc.\n" ,
   "\n" ,
   "At the same time, the routine in AFNI that initializes certain internal\n" ,
   "constants from X11 resources (usually in your .Xdefaults or .Xresources\n" ,
   "file, and described in file AFNI.Xdefaults) has been modified to also\n" ,
   "allow the same constants to be set from Unix environment variables.\n" ,
   "For example, the gap (in pixels) between sub-graphs is set by the\n" ,
   "X11 resource \"AFNI*graph_ggap\", and can now be set by the environment\n" ,
   "variables \"AFNI_graph_ggap\" or \"AFNI_GRAPH_GGAP\", as in\n" ,
   "\n" ,
   "  AFNI_graph_ggap = 6   // this is a comment\n" ,
   "\n" ,
   "If an X11 resource is actually set, it will take priority over the\n" ,
   "environment variable.  Some of the variables that can be set in this\n" ,
   "way are:\n" ,
   "\n" ,
   " AFNI_ncolors             = number of gray levels to use\n" ,
   " AFNI_gamma               = gamma correction for image intensities\n" ,
   " AFNI_graph_boxes_thick   = 0=thin lines, 1=thick lines, for graph boxes\n" ,
   " AFNI_graph_grid_thick    = ditto for the graph vertical grid lines\n" ,
   " AFNI_graph_data_thick    = ditto for the data graphs\n" ,
   " AFNI_graph_ideal_thick   = ditto for the ideal graphs\n" ,
   " AFNI_graph_ort_thick     = ditto for the ort graphs\n" ,
   " AFNI_graph_dplot_thick   = ditto for the dplot graphs\n" ,
   " AFNI_graph_ggap          = initial spacing between graph boxes\n" ,
   " AFNI_graph_matrix        = initial number of sub-graphs\n" ,
   " AFNI_fim_polort          = polynomial detrending order for FIM\n" ,
   " AFNI_fim_ignore          = how many pts to ignore at start when doing FIM\n" ,
   " AFNI_montage_periodic    = True allows periodic montage wraparound\n" ,
   " AFNI_purge               = True allows automatic dataset memory purge\n" ,
   " AFNI_resam_vox           = dimension of voxel (mm) for resampled datasets\n" ,
   " AFNI_resam_anat          = One of NN, Li, Cu, Bk for Anat resampling mode\n" ,
   " AFNI_resam_func          = ditto for Func resampling mode\n" ,
   " AFNI_resam_thr           = ditto for Threshold resampling mode\n" ,
   " AFNI_pbar_posfunc        = True will start color pbar as all positive\n" ,
   " AFNI_pbar_sgn_pane_count = # of panes to start signed color pbar with\n" ,
   " AFNI_pbar_pos_pane_count = # of panes to start positive color pbar with\n" ,
   "\n" ,
   "Some other such variables are described in file AFNI.Xdefaults.  Note that\n" ,
   "values that actually affect the way the X11/Motif interface appears, such as\n" ,
   "AFNI*troughColor, must be set via the X11 mechanism and cannot be set using\n" ,
   "Unix environment variables.  This is because they are interpreted by the\n" ,
   "Motif graphics library when it starts and not by any actual AFNI code.\n" ,
   "\n" ,
   "The following example is from my own .afnirc file on the Linux system on\n" ,
   "which I do most of the AFNI development.  The first ones (in lower case)\n" ,
   "are described in AFNI.Xdefaults.  The later ones (all upper case) are\n" ,
   "documented in this file.  (You can tell from this file that I like to\n" ,
   "have things line up.  You would never be able to tell this from the\n" ,
   "piles of paper in my office, though.)\n" ,
   "\n" ,
   " ***ENVIRONMENT\n" ,
   "\n" ,
   " AFNI_ncolors             = 60      // number of gray levels\n" ,
   " AFNI_gamma               = 1.5     // adjust for proper display\n" ,
   " AFNI_purge               = True    // purge datasets from memory when not used\n" ,
   " AFNI_chooser_doubleclick = Apply   // like Apply button; could also be Set\n" ,
   " AFNI_chooser_listmax     = 25      // max nonscrolling items in chooser lists\n" ,
   " AFNI_graph_width         = 512     // initial width of graph window (pixels)\n" ,
   " AFNI_graph_height        = 384     // initial height of graph window\n" ,
   " AFNI_graph_data_thick    = 1       // graph time series with thick lines\n" ,
   " AFNI_fim_ignore          = 2       // default value for FIM ignore\n" ,
   " AFNI_graph_ggap          = 7       // gap between sub-graphs (pixels)\n" ,
   " AFNI_pbar_hide           = True    // hide color pbar when it changes size\n" ,
   " AFNI_hotcolor            = Violet  // color to use on Done and Set buttons\n" ,
   " AFNI_SESSTRAIL           = 2       // see below for these ...\n" ,
   " AFNI_RENDER_ANGLE_DELTA  = 4.0     //                       |\n" ,
   " AFNI_RENDER_CUTOUT_DELTA = 4.0     //                       |\n" ,
   " AFNI_FIM_BKTHR           = 25.0    //                       |\n" ,
   " AFNI_SPLASHTIME          = 3.0     //                       v\n" ,
   "\n" ,
   "---------------------\n" ,
   "Variable: AFNI_ORIENT (editable)\n" ,
   "---------------------\n" ,
   "This is a string used to control the display of coordinates in the AFNI\n" ,
   "main control window.  The string must be 3 letters, one each from the\n" ,
   "pairs {R,L} {A,P} {I,S}.  The first letter in the string gives the\n" ,
   "orientation of the x-axis, the second the orientation of the y-axis,\n" ,
   "the third the z-axis:\n" ,
   "\n" ,
   "   R = right-to-left           L = left-to-right\n" ,
   "   A = anterior-to-posterior   P = posterior-to-anterior\n" ,
   "   I = inferior-to-superior    S = superior-to-inferior\n" ,
   "\n" ,
   "If AFNI_ORIENT is undefined, the default is RAI.  This is the order\n" ,
   "used by DICOM, and means\n" ,
   "\n" ,
   "   the -x axis is Right,    the +x axis is Left,\n" ,
   "   the -y axis is Anterior, the +y axis is Posterior,\n" ,
   "   the -z axis is Inferior, the +z axis is Superior.\n" ,
   "\n" ,
   "As a special case, using the code 'flipped' is equivalent to 'LPI',\n" ,
   "which is the orientation used in many neuroscience journals.\n" ,
   "\n" ,
   "This variable is also recognized by program 3dclust, which will report\n" ,
   "the cluster coordinates in the (x,y,z) order given by AFNI_ORIENT.\n" ,
   "Both AFNI and 3dclust also recognize the command line switch\n" ,
   "\"-orient string\", where string is a 3 letter code that can be used\n" ,
   "to override the value of AFNI_ORIENT.\n" ,
   "\n" ,
   "The plugin \"Coord Order\" (plug_coord.c) allows you to interactively\n" ,
   "change the orientation of the variable display within AFNI.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_PLUGINPATH\n" ,
   "-------------------------\n" ,
   "This variable should be the directory in which AFNI should search\n" ,
   "for plugins.  If there is more than one appropriate directory, they\n" ,
   "can be separated by colons, as in\n" ,
   "\n" ,
   "   setenv AFNI_PLUGINPATH /directory/one:/directory/two\n" ,
   "\n" ,
   "If this variable is not set, then AFNI will use the PATH variable\n" ,
   "instead.  This will waste time, since most directories in the PATH\n" ,
   "will not have plugins.  On some systems, using the PATH has been\n" ,
   "known to cause problems when AFNI starts.  I believe this is due to\n" ,
   "bugs in the system library routines (e.g., dlopen) used to manage\n" ,
   "dynamically loaded shared objects.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_NOPLUGINS\n" ,
   "------------------------\n" ,
   "If this YES/NO variable is set to YES, then AFNI will not try to\n" ,
   "read plugins when it starts up.  The command line switch \"-noplugins\"\n" ,
   "will have the same effect.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_YESPLUGOUTS\n" ,
   "--------------------------\n" ,
   "If this YES/NO variable is set to YES, then AFNI will try to listen\n" ,
   "for plugouts when it starts.  The command line switch \"-yesplugouts\"\n" ,
   "will have the same effect.  (Plugouts are an experimental feature\n" ,
   "that allow external programs to exchange data with AFNI.)  It is now\n" ,
   "also possible to start plugout listening from the Datamode->Misc menu.\n" ,
   "\n" ,
   "---------------------\n" ,
   "Variable: AFNI_TSPATH\n" ,
   "---------------------\n" ,
   "This variable should be set to any directory which you want to have\n" ,
   "AFNI scan for timeseries files (*.1D -- see the AFNI manual).  If\n" ,
   "more than one directory is desired, then colons can be used to\n" ,
   "separate them, as in AFNI_PLUGINPATH.  Note that timeseries files\n" ,
   "are read from all session directories, so directories provided by\n" ,
   "AFNI_TSPATH are designed to contain extra timeseries files that\n" ,
   "you want loaded no matter what AFNI sessions and datasets are being\n" ,
   "viewed.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_MODELPATH\n" ,
   "------------------------\n" ,
   "This variable should be set to the directory from which you want AFNI\n" ,
   "timeseries models to be loaded.  These models are similar to plugins,\n" ,
   "and are used by programs 3dNLfim, 3dTSgen, and the plugin plug_nlfit\n" ,
   "(menu label \"NLfit & NLerr\") -- see documentation file 3dNLfim.ps.\n" ,
   "If AFNI_MODELPATH is not given, then AFNI_PLUGINPATH will be used\n" ,
   "instead.\n" ,
   "\n" ,
   "-----------------------------------------\n" ,
   "Variable: AFNI_IMSIZE_* (or MCW_IMSIZE_*)\n" ,
   "-----------------------------------------\n" ,
   "These variables (named AFNI_IMSIZE_1 to AFNI_IMSIZE_99) allow you\n" ,
   "to control how the AFNI programs read binary image files.  The use of\n" ,
   "these is somewhat complicated, and is explained in detail at the end\n" ,
   "of the auxiliary programs manual (afni_aux.ps), in the section on \"3D:\"\n" ,
   "file specifications, and is also explained in the AFNI FAQ list.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_SESSTRAIL (editable)\n" ,
   "------------------------\n" ,
   "This variable controls the number of directory levels shown when\n" ,
   "choosing between session directories with the \"Switch Session\"\n" ,
   "button.  This variable should be set to a nonnegative integer.\n" ,
   "If a session directory name were\n" ,
   "   this/is/a/directory/name/\n" ,
   "then the \"Switch Session\" chooser would display the following:\n" ,
   "\n" ,
   "   AFNI_SESSTRAIL    Display\n" ,
   "   --------------    -------\n" ,
   "            0        name/\n" ,
   "            1        directory/name/\n" ,
   "            2        a/directory/name/\n" ,
   "            3        is/a/directory/name/\n" ,
   "            4        this/is/a/directory/name/\n" ,
   "\n" ,
   "That is, AFNI_SESSTRAIL determines how many trailing levels of\n" ,
   "the directory name are used for the display.  If AFNI_SESSTRAIL\n" ,
   "is not set, then it is equivalent to setting it to 0 (which\n" ,
   "was the old method).\n" ,
   "\n" ,
   "--------------------\n" ,
   "Variable: AFNI_HINTS\n" ,
   "--------------------\n" ,
   "This is a string controlling whether or not the popup \"hints\" are\n" ,
   "displayed when AFNI starts.  If the string is \"NO\", then the hints\n" ,
   "are disabled when AFNI starts, otherwise they are enabled.  In\n" ,
   "either case, they can be turned off and on interactively from the\n" ,
   "Define Datamode->Misc menu.\n" ,
   "\n" ,
   "Hints can be permanently disabled by setting the C macro\n" ,
   "DONT_USE_HINTS in machdep.h and recompiling AFNI.  They can also\n" ,
   "be disabled at runtime by setting AFNI_HINTS to \"KILL\".\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_COMPRESSOR (cf. AFNI_AUTOGZIP) (editable)\n" ,
   "-------------------------\n" ,
   "This variable is used to control automatic compression of .BRIK files on\n" ,
   "output.  The legal values are \"COMPRESS\", \"GZIP\", and \"BZIP2\", which\n" ,
   "respectively invoke programs \"compress\", \"gzip\", and \"bzip2\" (the program\n" ,
   "must be in your path for compression to work).  If AFNI_COMPRESSOR is\n" ,
   "equal to one of these, then all AFNI programs will automatically pass\n" ,
   ".BRIK data through the appropriate compression program as it is written\n" ,
   "to disk.  Note that this will slow down dataset write operations.  Note\n" ,
   "also that compressed datasets cannot be mapped directly from disk into\n" ,
   "memory ('mmap'), but must occupy actual memory (RAM) and swap space.  For\n" ,
   "more details, see file README.compression.\n" ,
   "\n" ,
   "Note that compressed (.BRIK.Z, .BRIK.gz, and .BRIK.bz2) datasets will\n" ,
   "automatically be uncompressed on input, no matter what the setting of\n" ,
   "this variable.  AFNI_COMPRESSOR only controls how the datasets are\n" ,
   "written.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_BYTEORDER\n" ,
   "------------------------\n" ,
   "This variable is used to control the byte order for output files.\n" ,
   "If you use it, the two legal values are \"LSB_FIRST\" and \"MSB_FIRST\".\n" ,
   "If you don't use it, the default order on your CPU will be used.\n" ,
   "The main purpose of this would be if you were using a mixture of\n" ,
   "CPU types reading shared disks (i.e., using NFS).  If the majority\n" ,
   "of the systems were MSB_FIRST (e.g., SGI, HP, Sun), but there were\n" ,
   "a few LSB_FIRST systems (e.g., Intel, DEC Alpha), then you might\n" ,
   "want to do 'setenv AFNI_BYTEORDER MSB_FIRST' on all of the MSB_FIRST\n" ,
   "systems to make sure that the datasets that they write out are\n" ,
   "readable by the other computers.\n" ,
   "\n" ,
   "Note that AFNI programs can now check the .HEAD file for the byte\n" ,
   "order of a dataset, and will swap the bytes on input, if needed.\n" ,
   "If you wish to mark all of the datasets on a given system as\n" ,
   "being in a particular order, the following command should work:\n" ,
   "\n" ,
   " find /top/dir -name \\*.HEAD -exec 3drefit -byteorder NATIVE_ORDER {} \\;\n" ,
   "\n" ,
   "Here, '/top/dir' is the name of the top level directory under\n" ,
   "which you wish to search for AFNI datasets.  The string NATIVE_ORDER\n" ,
   "means to set all datasets to the CPU default order, which is probably\n" ,
   "what you are using now.  (You can use the program 'byteorder' to\n" ,
   "find out the native byte ordering of your CPU.)\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_BYTEORDER_INPUT\n" ,
   "------------------------------\n" ,
   "This variable is used to control the byte order for input files.\n" ,
   "If you use it, the two legal values are \"LSB_FIRST\" and \"MSB_FIRST\".\n" ,
   "The value of this variable is only used for old datasets that do\n" ,
   "not have the byte order encoded in their headers.  If this variable\n" ,
   "is not present, then the CPU native byte order is used.  If this\n" ,
   "variable is present, and its value differs from the native byte\n" ,
   "order, then 2 byte dataset BRIKs (short) are 2-swapped (as in\n" ,
   "\"ab\" -> \"ba\") when they are read from disk, and 4 byte datasets\n" ,
   "(float, complex) are 4-swapped (\"abcd\" -> \"dcba\").\n" ,
   "\n" ,
   "[per the request of John Koger]\n" ,
   "\n" ,
   "---------------------\n" ,
   "Variable: AFNI_NOMMAP\n" ,
   "---------------------\n" ,
   "This YES/NO variable can be used to turn off the mmap feature by which\n" ,
   "AFNI can load datasets into memory using the map-file-to-memory\n" ,
   "functionality of Unix.  (Dataset .BRIK files will only be mmap-ed if\n" ,
   "they are not compressed and are in the native byte order of the CPU.)\n" ,
   "On some systems, mmap doesn't seem to work very well (e.g., Linux kernel\n" ,
   "version 1.2.13).  You can disable mmap by 'setenv AFNI_NOMMAP YES'.\n" ,
   "\n" ,
   "The penalty for disabling mmap is that all datasets must be loaded\n" ,
   "into actual RAM.  AFNI does not have the ability to load a dataset\n" ,
   "only partially, so if a 20 Megabyte .BRIK file is accessed, all of\n" ,
   "it will be loaded into RAM.  With mmap, the Unix operating system will\n" ,
   "decide how much of the file to load.  In this way, it is possible to\n" ,
   "deal with more files than you have swap space on your computer\n" ,
   "(since .BRIK files are mmap-ed in readonly mode).\n" ,
   "\n" ,
   "The moral of the story: buy more memory, it's cheap.  At the time\n" ,
   "I write this line [Aug 1998], I have a PC with 384 MB of RAM, and\n" ,
   "it is great to use with AFNI.\n" ,
   "\n" ,
   "[Feb 2004] I now have a Mac G5 with 8 GB of RAM, and it is even greater!\n" ,
   "\n" ,
   "----------------------\n" ,
   "Variable: AFNI_PSPRINT (editable)\n" ,
   "----------------------\n" ,
   "This variable is used to define a command that will print the\n" ,
   "standard input (stdin) to a PostScript printer.  If it is defined,\n" ,
   "the \"->printer\" button on the timeseries \"Plot\" windows will work.\n" ,
   "For some Unix systems, the following should work:\n" ,
   "  setenv AFNI_PSPRINT \"lp -\"\n" ,
   "For others, this may work\n" ,
   "  setenv AFNI_PSPRINT \"lpr -\"\n" ,
   "It all depends on the printer software setup you have.  To send the\n" ,
   "output into GhostView\n" ,
   "  setenv AFNI_PSPRINT \"ghostview -landscape -\"\n" ,
   "\n" ,
   "In the (very far distant) future, other windows (e.g., image and graph\n" ,
   "displays) may get the ability to print to a PostScript file or printer.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_LEFT_IS_LEFT (editable)\n" ,
   "---------------------------\n" ,
   "Setting this YES/NO variable to YES tells AFNI to display images with\n" ,
   "the left side of the subject on the left side of the window.  The default\n" ,
   "mode is to display the right side of the subject on the left side of\n" ,
   "the window - the radiology convention.  This \"handedness\" can also be\n" ,
   "controlled with the \"-flipim\" and \"-noflipim\" command line options to AFNI.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_ALWAYS_LOCK\n" ,
   "--------------------------\n" ,
   "Setting this YES/NO variable to YES tells AFNI to start up with all\n" ,
   "the controller windows locked together.  If you mostly use multiple\n" ,
   "controllers to view datasets in unison, then this will be useful.\n" ,
   "Notice that the Time Lock feature is not automatically enabled\n" ,
   "by this -- you must still actuate it manually from the Lock menu\n" ,
   "on the Define Datamode panel.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variables: AFNI_RENDER_* (editable)\n" ,
   "------------------------\n" ,
   "These variables set some defaults in the \"Render Dataset\" (volume\n" ,
   "rendering) plugin.  The first two variables are\n" ,
   "\n" ,
   "  AFNI_RENDER_ANGLE_DELTA  = stepsize for viewing angles, in degrees\n" ,
   "  AFNI_RENDER_CUTOUT_DELTA = stepsize for cutout dimensions, in mm\n" ,
   "\n" ,
   "These stepsizes control how much the control parameters change when\n" ,
   "one of their up- or down-arrows is pressed.  Both of these stepsize\n" ,
   "values default to 5.0.\n" ,
   "\n" ,
   "The third variable is\n" ,
   "\n" ,
   "  AFNI_RENDER_PRECALC_MODE = \"Low\", \"Medium\", or \"High\"\n" ,
   "\n" ,
   "This is used to set the initial precalculation mode for the renderer\n" ,
   "(this mode can be altered interactively, unlike the stepsizes).\n" ,
   "\n" ,
   "The fourth variable is\n" ,
   "\n" ,
   "  AFNI_RENDER_SHOWTHRU_FAC = some number between 0.0 and 1.0\n" ,
   "\n" ,
   "This is used to control the way in which the \"ShowThru\" Color Opacity\n" ,
   "option renders images.  See the rendering plugin Help window for more\n" ,
   "information.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_NOREALPATH\n" ,
   "-------------------------\n" ,
   "Normally, when AFNI reads a list of session directories, it converts\n" ,
   "their names to the \"real path\" form, which follows symbolic links, and\n" ,
   "removes '/./' and '/../' components.  These converted names are used\n" ,
   "for display purposes in the \"Switch Session\" chooser and in other\n" ,
   "places.  If you wish to have the names NOT converted to the \"real path\"\n" ,
   "format, set this YES/NO environment variable to YES, as in\n" ,
   "\n" ,
   "   setenv AFNI_NOREALPATH YES\n" ,
   "\n" ,
   "(For more information on the \"real path\" conversion, see the Unix\n" ,
   "man page for the realpath() function.)  Note that if you use this\n" ,
   "feature, then the effect of AFNI_SESSTRAIL will be limited to what\n" ,
   "you type on the command line, since it is the realpath() function\n" ,
   "that provides the higher level hierarchies of the session names.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_NO_MCW_MALLOC\n" ,
   "----------------------------\n" ,
   "AFNI uses a set of \"wrapper\" macros and functions to let itself keep\n" ,
   "track of the memory allocated and freed by the C malloc() library.\n" ,
   "This is useful for debugging purposes (see the last items on the 'Misc'\n" ,
   "menu in the AFNI 'Define Datamode' control panel), but carries a small\n" ,
   "overhead (both in memory and speed).  Setting this YES/NO environment\n" ,
   "variable to YES provides one way to disable this facility, as in\n" ,
   "\n" ,
   "   setenv AFNI_NO_MCW_MALLOC YES\n" ,
   "\n" ,
   "Another way to permanently disable this capability (so that it isn't\n" ,
   "even compiled) is outlined in the file machdep.h.  Also, the interactive\n" ,
   "AFNI program takes the command line switch \"-nomall\", which will turn\n" ,
   "off these functions for the given run.\n" ,
   "\n" ,
   "N.B.: Setting this variable in the .afnirc file will have no effect,\n" ,
   "      since the decision whether to use the routines in mcw_malloc.c\n" ,
   "      is made at the very start of the program, before .afnirc is\n" ,
   "      scanned.  Therefore, to use this variable, you must set it\n" ,
   "      externally, perhaps in your .cshrc or .profile initialization\n" ,
   "      file.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_FIM_BKTHR\n" ,
   "------------------------\n" ,
   "This sets the threshold for the elimination of the background voxels\n" ,
   "during the interactive FIM calculations.  The average intensity of\n" ,
   "all voxels in the first 3D volume used in the correlation is calculated.\n" ,
   "Voxels with intensity below 0.01 * AFNI_FIM_BKTHR * (this average)\n" ,
   "will not have the correlation computed.  The default value is 10.0, but\n" ,
   "values as large as 50.0 may be useful.  This parameter may be changed\n" ,
   "interactively from the FIM->Edit Ideal submenu in a graph viewer.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_FLOATSCAN (editable)\n" ,
   "------------------------\n" ,
   "If this YES/NO variable is set to YES, then floating point bricks\n" ,
   "are checked for illegal values (NaN and Infinity) when they are\n" ,
   "read into an AFNI program -- illegal values will be replaced by\n" ,
   "zeros.  If a dataset brick contains such illegal values that go\n" ,
   "undetected, AFNI programs will probably fail miserably, and have\n" ,
   "been known to go into nearly-infinite loops.\n" ,
   "\n" ,
   "Setting this variable implies setting AFNI_NOMMAP to YES, since\n" ,
   "only in-memory bricks can be altered (mmap-ed bricks are readonly).\n" ,
   "\n" ,
   "The command line program 'float_scan' can be used to check and\n" ,
   "patch floating point files.\n" ,
   "\n" ,
   "[14 Sep 1999] The program to3d will scan input float and complex\n" ,
   "files for illegal values, and patch illegal input numbers with\n" ,
   "zeros in the output dataset.  If this behavior is not desired for\n" ,
   "some bizarre reason, the '-nofloatscan' command line option to\n" ,
   "to3d must be used.\n" ,
   "\n" ,
   "-----------------------\n" ,
   "Variable: AFNI_NOSPLASH\n" ,
   "-----------------------\n" ,
   "If this YES/NO variable is set to YES, then the AFNI splash screen\n" ,
   "will not be displayed when the program starts.  I'm not sure WHY\n" ,
   "you would want to disable this thing of beauty (which is a joy\n" ,
   "forever), but if your soul is thusly degraded, so be it.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_SPLASH_XY\n" ,
   "------------------------\n" ,
   "If set, this variable should be in the form \"100:37\" (two integers\n" ,
   "separated by a colon).  These values specify the (x,y) screen location\n" ,
   "where the splash window's upper left corner will be placed.  If not\n" ,
   "set, x will be set to center the splash window on the display and\n" ,
   "y will be 100.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_SPLASHTIME\n" ,
   "-------------------------\n" ,
   "The value of this variable determines how long the AFNI splash screen\n" ,
   "will stay popped up, in seconds (default value = 5.0).  The splash\n" ,
   "screen will always stay up until the first AFNI controller window is\n" ,
   "ready for use.  If the time from program start to this ready condition\n" ,
   "is less than AFNI_SPLASHTIME, the splash screen will stay up until\n" ,
   "AFNI_SPLASHTIME has elapsed; otherwise, the splash screen will be\n" ,
   "removed as soon as AFNI is ready to go.  By setting AFNI_SPLASHTIME\n" ,
   "to 0.0, you can have the splash screen removed as soon as possible\n" ,
   "(and the fade-out feature will be disabled).\n" ,
   "\n" ,
   "--------------------------------\n" ,
   "Variable: AFNI_FIM_PERCENT_LIMIT (editable)\n" ,
   "--------------------------------\n" ,
   "This sets an upper limit on the % Change that the FIM+ computation\n" ,
   "will compute.  For example\n" ,
   "\n" ,
   "  setenv AFNI_FIM_PERCENT_LIMIT 50\n" ,
   "\n" ,
   "means that computed values over 50% will be set to 50%, and values\n" ,
   "below -50% will be set to -50%.  This can be useful to avoid scaling\n" ,
   "problems that arise when some spurious voxels with tiny baselines have\n" ,
   "huge percent changes.  This limit applies to all 3 possible percentages\n" ,
   "that FIM and FIM+ can compute: % from baseline, % from average, and\n" ,
   "% from top.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_NOTES_DLINES\n" ,
   "---------------------------\n" ,
   "This sets the upper limit on the number of lines displayed in the\n" ,
   "Notes plugin, for each note.  If not present, the limit is 9 lines\n" ,
   "shown per note at once.  To see a note longer than this limit, you'll\n" ,
   "have to use the vertical scrollbar.\n" ,
   "\n" ,
   "-----------------------\n" ,
   "Variable: AFNI_FIM_MASK\n" ,
   "-----------------------\n" ,
   "This chooses the default subset of values computed with the FIM+\n" ,
   "button in a graph window.  The mask should be the sum of the desired\n" ,
   "values from this list:\n" ,
   "\n" ,
   "    1 = Fit Coef\n" ,
   "    2 = Best Index\n" ,
   "    4 = % Change\n" ,
   "    8 = Baseline\n" ,
   "   16 = Correlation\n" ,
   "   32 = % From Ave\n" ,
   "   64 = Average\n" ,
   "  128 = % From Top\n" ,
   "  256 = Topline\n" ,
   "  512 = Sigma Resid\n" ,
   "\n" ,
   "If you don't set this variable, the default mask is 23 = 1+2+4+16.\n" ,
   "\n" ,
   "-----------------------------------\n" ,
   "Variable: AFNI_NO_BYTEORDER_WARNING\n" ,
   "-----------------------------------\n" ,
   "If this YES/NO variable is set to YES, then AFNI program will not\n" ,
   "warn you when reading in a dataset that does not contain a byte\n" ,
   "order flag.  The default is to issue such a warning.  Only older\n" ,
   "versions of AFNI create datasets that don't have the byte order\n" ,
   "flag.  (See also the variable AFNI_BYTEORDER, described far above.)\n" ,
   "The purpose of this warning is to alert you to possible problems\n" ,
   "when you move datasets between computers with different CPU types.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_PCOR_DENEPS\n" ,
   "--------------------------\n" ,
   "The correlation coefficient calculated in FIM is calculated as the\n" ,
   "ratio of two quantities.  If the denominator is negative or zero,\n" ,
   "then this value is meaningless and may even cause the program to\n" ,
   "crash.  Mathematically, the denominator cannot be zero or negative,\n" ,
   "but this could arise due to finite precision arithmetic on the computer\n" ,
   "(i.e., roundoff error accumulation).  To avoid this problem, the routine\n" ,
   "that computes the correlation coefficient compares the denominator to a\n" ,
   "value (called DENEPS) - if the denominator is less than DENEPS, then\n" ,
   "the correlation coefficient for that voxel is set to zero.\n" ,
   "\n" ,
   "The denominator that is being computed is proportional to the variance\n" ,
   "of the time series.  If the voxel time series data is very small, then\n" ,
   "the variance will be really small - so much so that the DENEPS test\n" ,
   "will be failed, even though it shouldn't be.  This problem has arisen\n" ,
   "when people input time series whose typical value is 0.001 or smaller.\n" ,
   "It never occurred to me that people would input data this small to the\n" ,
   "AFNI FIM routines.  To get around this difficulty, set this environment\n" ,
   "variable to a value for DENEPS; for example\n" ,
   "  setenv AFNI_PCOR_DENEPS 0.0\n" ,
   "will turn off the checking entirely.  Or you could do\n" ,
   "  setenv AFNI_PCOR_DENEPS 1.e-10\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_ENFORCE_ASPECT (editable)\n" ,
   "-----------------------------\n" ,
   "Some Linux window managers do no enforce the aspect ratio (width to height\n" ,
   "proportion) request that the image display module makes.  This means that\n" ,
   "image windows can become undesirably distorted when manually resized.\n" ,
   "Setting this YES/NO variable to YES will make AFNI itself enforce the\n" ,
   "aspect ratio whenever an image window is resized.\n" ,
   "\n" ,
   "----------------------------------------\n" ,
   "Variables: AFNI_<plug_filename>_butcolor\n" ,
   "----------------------------------------\n" ,
   "These variables (one for each AFNI plugin) let you set the menu button colors\n" ,
   "for the Plugins menu item.  For example\n" ,
   "  setenv AFNI_plug_power_butcolor red3\n" ,
   "will make the \"Power Spectrum\" button appear in a dark red color.  The format\n" ,
   "of the variable is exemplified above: the <plug_filename> is replaced by the\n" ,
   "filename of the plugin (after removing the suffix).  Note that it is possible\n" ,
   "for the plugin author to hardcode the menu button for his/her plugin, in\n" ,
   "which case the corresponding environment variable will have no effect.\n" ,
   "\n" ,
   "Colors are specified as described in file README.setup.  If you are using\n" ,
   "an X11 PseudoColor visual, then you should be economical with color usage!\n" ,
   "\n" ,
   "The purpose of this feature is to let you highlight the plugins that you\n" ,
   "use most frequently.  The size of the of plugin menu is growing, and it\n" ,
   "is easy to misplace what you most use in the list.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_MARKERS_NOQUAL (editable)\n" ,
   "-----------------------------\n" ,
   "If this YES/NO variable is set to YES, then the interactive AFNI program\n" ,
   "behaves as if the \"-noqual\" command line option had been included.  This\n" ,
   "feature was added at the request of Dr. Michael S. Beauchamp, who has a\n" ,
   "very rare neurological disorder called \"noqaulagnosia\".\n" ,
   "\n" ,
   "----------------------\n" ,
   "Variable: AFNI_OPTIONS\n" ,
   "----------------------\n" ,
   "In the spirit of the previous variable, this variable can be used to set\n" ,
   "up command line options that will always be passed to the interactive\n" ,
   "AFNI program.  If more than one option is needed, then they should be\n" ,
   "separated by spaces, and the whole value of the variable will need to be\n" ,
   "placed in quotes.  For example\n" ,
   "\n" ,
   "   setenv AFNI_OPTIONS \"-noqual -ncolors 60\"\n" ,
   "\n" ,
   "Note that the AFNI command line option \"-nomall\" cannot be specified this\n" ,
   "way (cf. the discussion under variable AFNI_NO_MCW_MALLOC).\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_NO_SIDES_LABELS (editable)\n" ,
   "------------------------------\n" ,
   "As of 01 Dec 1999, the interactive AFNI program now displays a label\n" ,
   "beneath each image window showing which side of the image is on the left\n" ,
   "edge of the window.  This label is based on the anatomical directions\n" ,
   "encoded in the anatomical dataset .HEAD file, usually when to3d was used\n" ,
   "to create the file.  If you do NOT want these labels displayed (why not?),\n" ,
   "set this YES/NO environment variable to YES.\n" ,
   "\n" ,
   "----------------------------------\n" ,
   "Variable: AFNI_NO_ADOPTION_WARNING\n" ,
   "----------------------------------\n" ,
   "AFNI now can print a warning when it forces a dataset to have an anatomy\n" ,
   "parent dataset (the \"forced adoption\" function).  This happens when\n" ,
   "there a dataset does not have an anatomy parent encoded into its .HEAD\n" ,
   "file (either via to3d or 3drefit), and there is more than one anatomical\n" ,
   "dataset in the directory that has Talairach transformation markers\n" ,
   "attached.  If you wish to enable this warning, set this YES/NO variable\n" ,
   "to NO.  For more information on this subject, please see\n" ,
   "  http://afni.nimh.nih.gov/afni/afni_faq.shtml#AnatParent .\n" ,
   "\n" ,
   "-----------------------------------\n" ,
   "Variable: AFNI_NO_NEGATIVES_WARNING\n" ,
   "-----------------------------------\n" ,
   "If this YES/NO variable is set to YES, then to3d will skip the usual\n" ,
   "warning that it pops up in a message window when it discovers negative\n" ,
   "values in the input short images.  (The warning will still be printed\n" ,
   "to stdout.)\n" ,
   "\n" ,
   "----------------------\n" ,
   "Variable: AFNI_NO_XDBE\n" ,
   "----------------------\n" ,
   "If this YES/NO variable is set to YES, then the X11 Double Buffer\n" ,
   "Extension (XDBE) will not be used, even if the X11 server supports it.\n" ,
   "This is needed when the X11 server says that it supports it, but actually\n" ,
   "does not implement it correctly - this is a problem on the Xsgi server\n" ,
   "running under IRIX 6.5.3 on R4400 machines.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_VIEW_ANAT_BRICK (editable)\n" ,
   "          AFNI_VIEW_FUNC_BRICK (editable)\n" ,
   "------------------------------\n" ,
   "One of the (very few) confusing parts of AFNI is the \"warp-on-demand\"\n" ,
   "viewing of transformed datasets (e.g., in the +tlrc coordinate system).\n" ,
   "This allows you to look at slices taken from transformed volumes without\n" ,
   "actually computing and storing the entire transformed dataset.  This\n" ,
   "viewing mode is controlled by from the \"Define Datamode\" control panel.\n" ,
   "When an anatomical dataset has a +tlrc.BRIK file, then you can choose\n" ,
   "between \"View Anat Data Brick\" and \"Warp Anat on Demand\"; when there\n" ,
   "is no +tlrc.BRIK file for the dataset, then only \"Warp Anat on Demand\"\n" ,
   "is possible.\n" ,
   "\n" ,
   "If you switch the Talairach view when the current anat dataset does\n" ,
   "not have a +tlrc.BRIK file, then the \"Warp Anat on Demand\" mode will\n" ,
   "be turned on.  If you then switch to a dataset that does have a\n" ,
   "+tlrc.BRIK file, \"Warp Anat on Demand\" will still be turned on,\n" ,
   "although the \"View Anat Data Brick\" option will be enabled.\n" ,
   "\n" ,
   "If you set the YES/NO variable AFNI_VIEW_ANAT_BRICK to YES,\n" ,
   "then \"View Anat Data Brick\" will be turned on whenever possible after\n" ,
   "switching datasets.  Similarly, setting AFNI_VIEW_FUNC_BRICK to YES\n" ,
   "will engage \"View Func Data Brick\" whenever possible (when the BRIK\n" ,
   "file exists and its grid spacing matches the anatomical grid spacing).\n" ,
   "Note that switching any dataset (func or anat) triggers the same\n" ,
   "routine, and will set either or both \"View Brick\" modes on.  When\n" ,
   "these environment variables are present, the only way to switch to\n" ,
   "\"Warp\" mode when \"View Brick\" mode is possible is to do it manually\n" ,
   "(by clicking on the toggle button) when you want this.\n" ,
   "\n" ,
   "When you use one of the drawing plugins (\"Draw Dataset\" or \"Gyrus Finder\"),\n" ,
   "you must operate directly on the dataset BRIK.  For this reason, it is\n" ,
   "important to be in \"View Data Brick\" mode on these occasions.  Setting\n" ,
   "these variables is one way to ensure that this will happen whenever\n" ,
   "possible.\n" ,
   "\n" ,
   "When AFNI is in \"Warp Anat on Demand\" mode, the word \"{warp}\" will\n" ,
   "appear in the windows' titlebars.  This provides a reminder of the\n" ,
   "viewing mode you are using (warped from a brick, or data directly\n" ,
   "extracted from a brick), since the \"Define Datamode\" control panel\n" ,
   "will not always be open.\n" ,
   "\n" ,
   "08 Aug 2003: I have modified the way these variables are treated in\n" ,
   "AFNI so that they now default to the \"YES\" behavior.  If you don't\n" ,
   "want this, you have to explicitly set them to \"NO\" from this day forth.\n" ,
   "\n" ,
   "----------------\n" ,
   "Variable: TMPDIR\n" ,
   "----------------\n" ,
   "This variable specifies the directory where temporary files are to be\n" ,
   "written.  It is not unique to AFNI, but is used by many Unix programs.\n" ,
   "You must have permission to write into this directory.  If you want to\n" ,
   "use the current directory, setting TMPDIR to \".\" would work.  If TMPDIR\n" ,
   "is not defined, directory /tmp will be used.  On some systems, this\n" ,
   "directory may not have enough space for the creation of large temporary\n" ,
   "datasets.  On most Unix systems, you can tell the size of various disk\n" ,
   "partitions using a command like \"df -k\" (on HPUX, \"bdf\" works).\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_GRAYSCALE_BOT\n" ,
   "----------------------------\n" ,
   "This variable sets the darkest level shown in a grayscale image window.\n" ,
   "The default value is 55 (a leftover from Andrzej Jesmanowicz).  You can\n" ,
   "set this value to anything from 0 to 254.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_SYSTEM_AFNIRC\n" ,
   "----------------------------\n" ,
   "If this variable is set, it is the name of a file to be read like the\n" ,
   "user's .afnirc file (see README.setup).  The purpose is to allow a\n" ,
   "system-wide setup file to be used.  To do this, you would create such\n" ,
   "a file in a useful place - perhaps where you store the AFNI binaries.\n" ,
   "Then each user account should have the equivalent of\n" ,
   "\n" ,
   "   setenv AFNI_SYSTEM_AFNIRC /place/where/setup/is/stored/.afnirc\n" ,
   "\n" ,
   "defined in its .cshrc (.bashrc, etc.) file.  Note that it doesn't make\n" ,
   "sense to define this variable in the user's .afnirc file, since that\n" ,
   "file won't be read until AFTER this file is read.  Also note that use\n" ,
   "of the -skip_afnirc option will cause both the system and user setup\n" ,
   "files to be skipped.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_PBAR_IMXY (editable)\n" ,
   "------------------------\n" ,
   "This variable determines the size of the image saved when the\n" ,
   "\"Save to PPM\" button is selected for a color pbar.  It should be\n" ,
   "in the format\n" ,
   "  setenv AFNI_PBAR_IMXY 20x256\n" ,
   "which means to set the x-size (horizontal) to 20 pixels and the\n" ,
   "y-size (vertical) to 256 pixels.  These values are the default,\n" ,
   "by the way.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_LAYOUT_FILE\n" ,
   "--------------------------\n" ,
   "If defined, this variable is the name of a file to read at startup\n" ,
   "to define the \"layout\" of AFNI windows at the program start.  If\n" ,
   "this name starts with a '/' character, then it is an absolute path;\n" ,
   "otherwise, it is taken to be a path relative to the user's home\n" ,
   "directory ($HOME).  If the AFNI command line switch \"-layout\" is\n" ,
   "used, it will override this specification.\n" ,
   "\n" ,
   "The simplest way to produce a layout file is to use the \"Save Layout\"\n" ,
   "button on the Datamode->Misc menu.  You can then edit this file;\n" ,
   "the format should be fairly self-explanatory.  The structure of the\n" ,
   "file is similar to the .afnirc file (cf.  README.setup).  In fact,\n" ,
   "the layout file can be included into .afnirc (since it is just another\n" ,
   "*** section) and then setting AFNI_LAYOUT_FILE = .afnirc in the\n" ,
   "***ENVIRONMENT section should work.\n" ,
   "\n" ,
   "A sample layout file:\n" ,
   "\n" ,
   "***LAYOUT\n" ,
   " A               geom=+73+1106                 // start controller A\n" ,
   " A.sagittalimage geom=320x320+59+159 ifrac=0.8 // and Sagittal image\n" ,
   " A.sagittalgraph geom=570x440+490+147 matrix=9 // and Sagittal graph\n" ,
   " B                                             // start controller B\n" ,
   " B.plugin.ROI_Average                          // start a plugin\n" ,
   "\n" ,
   "Each window to be opened has a separate command line in this file.\n" ,
   "The \"geom=\" qualifiers specify the size and position of the windows.\n" ,
   "For images, \"ifrac=\" can be used to specify the fraction of the window\n" ,
   "occupied by the image (if \"ifrac=1.0\", then no control widgets will be\n" ,
   "visible).  For graphs, \"matrix=\" can be used to control the initial\n" ,
   "number of sub-graphs displayed.  For plugins, the label on the button\n" ,
   "that starts the plugin is used after the \".plugin.\" string (blanks\n" ,
   "should be filled with underscores \"_\").  In the example above, the last\n" ,
   "two windows to be opened do not have a \"geom=\" qualifier, so their\n" ,
   "placement will be chosen by the window manager.\n" ,
   "\n" ,
   "If you add \"slow\" after the \"***LAYOUT\", then each window operation\n" ,
   "will be paused for 1 second to let you watch the layout operations\n" ,
   "proceed gradually.  Otherwise, they will be executed as fast as\n" ,
   "possible (which still may not be all that fast).\n" ,
   "\n" ,
   "Using layouts with a window manager that requires user placement\n" ,
   "of new windows (e.g., twm) is a futile and frustrating exercise.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_tsplotgeom\n" ,
   "-------------------------\n" ,
   "Related to the above, if you set this environment variable (in the\n" ,
   "***ENVIRONMENT section, not in the ***LAYOUT section), it is used\n" ,
   "to set the geometry of the plotting windows used for time series\n" ,
   "plots, histograms, etc. -- all the graphs except the dataset plots.\n" ,
   "Its format should be something like \"550x350\"; this example sets\n" ,
   "the width to 550 pixels and the height to 350 pixels.  If you don't\n" ,
   "set this, the default is \"200x200\", which is quite small on a high\n" ,
   "resolution display.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variables: AFNI_REALTIME_*\n" ,
   "--------------------------\n" ,
   "This set of variables allows you to control the initial setup of the\n" ,
   "realtime data acquisition plugin (menu item \"RT Options\").  Normally,\n" ,
   "this plugin is active only if AFNI is started with the \"-rt\" command\n" ,
   "line option.  (It will consume CPU time continually as it polls for\n" ,
   "an incoming data connection, which is why you don't want it running\n" ,
   "by default.)  The following variables can be used to initialize the\n" ,
   "plugin's options:\n" ,
   "\n" ,
   "AFNI_REALTIME_Activate = This is a YES/NO variable, and allows you\n" ,
   "                         to have the realtime plugin active without\n" ,
   "                         using the \"-rt\" command line option.  If\n" ,
   "                         this variable is set to YES, then you can\n" ,
   "                         disable the realtime plugin with \"-nort\".\n" ,
   "\n" ,
   "The variables below are used to set the initial status of the widgets\n" ,
   "in the realtime plugin's control window.  Each one has the same name as\n" ,
   "the labels in the control window, with blanks replaced by underscores.\n" ,
   "The values to set for these variables are exact copies of the inputs\n" ,
   "you would specify interactively (again, with blanks replaced by\n" ,
   "underscores).  For details as to the meaning of these options, see\n" ,
   "the plugin's Help window.\n" ,
   "\n" ,
   "AFNI_REALTIME_Images_Only  = \"No\" or \"Yes\"\n" ,
   "AFNI_REALTIME_Root         = name for datasets to be created\n" ,
   "AFNI_REALTIME_Update       = an integer from 0 to 19\n" ,
   "AFNI_REALTIME_Function     = \"None\" or \"FIM\" (cf. AFNI_FIM_IDEAL below)\n" ,
   "AFNI_REALTIME_Verbose      = \"No\", \"Yes\", or \"Very\"\n" ,
   "AFNI_REALTIME_Registration = \"None\", \"2D:_realtime\", \"2D:_at_end\",\n" ,
   "                             \"3D:_realtime\", \"3D:_at_end\",\n" ,
   "                             or \"3D:_estimate\"\n" ,
   "AFNI_REALTIME_Base_Image   = an integer from 0 to 59\n" ,
   "AFNI_REALTIME_Resampling   = \"Cubic\", \"Quintic\", \"Heptic\", \"Fourier\",\n" ,
   "                             or \"Hept+Four\"\n" ,
   "AFNI_REALTIME_Graph        = \"No\", \"Yes\", or \"Realtime\"\n" ,
   "AFNI_REALTIME_NR           = an integer from 5 to 9999\n" ,
   "AFNI_REALTIME_YR           = a floating point number from 0.1 to 10.0\n" ,
   "\n" ,
   "The following internal controls can only be set using these environment\n" ,
   "variables (there is no GUI to set these values):\n" ,
   "\n" ,
   "AFNI_REALTIME_volreg_maxite      = an integer >= 1 [default = 9]\n" ,
   "AFNI_REALTIME_volreg_maxite_est  = an integer >= 1 [default = 1]\n" ,
   "AFNI_REALTIME_volreg_graphgeom   = something like 320x320+59+159\n" ,
   "\n" ,
   "AFNI_REALTIME_CHILDWAIT = max wait time (in sec) for child info\n" ,
   "                          process [default = 66.6]; not needed if\n" ,
   "                          child info process is not used\n" ,
   "\n" ,
   "AFNI_REALTIME_WRITEWAIT = if the image data pauses for this number\n" ,
   "                          of seconds, then the datasets being constructed\n" ,
   "                          will be written to disk [default=37.954];\n" ,
   "                          since this output may take several seconds,\n" ,
   "                          you may need to adjust this if you are in\n" ,
   "                          fact doing imaging with a very long TR.\n" ,
   "                          Note that after this wait, the plugin can\n" ,
   "                          still receive image data -- even if the image\n" ,
   "                          source program is silent for a very long time,\n" ,
   "                          AFNI will still be waiting patiently for data.\n" ,
   "\n" ,
   "AFNI_GRAPH_AUTOGRID     = By default, if the number of time points in an\n" ,
   "                          AFNI graph viewer changes, the density of\n" ,
   "                          vertical grid lines changes.  If you don't\n" ,
   "                          want this to happen, set this variable to NO.\n" ,
   "\n" ,
   "For more information about how the realtime plugin works, read the file\n" ,
   "README.realtime.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_FIM_IDEAL\n" ,
   "------------------------\n" ,
   "This variable specifies the filename of the initial FIM ideal timeseries.\n" ,
   "The main use of this would be to be able to initialize the Realtime\n" ,
   "plugin without direct user intervention.\n" ,
   "\n" ,
   "----------------------------------\n" ,
   "Variable: AFNI_PLUGINS_ALPHABETIZE\n" ,
   "----------------------------------\n" ,
   "If this YES/NO variable is set to NO, then the plugin buttons will\n" ,
   "not be alphabetized on the menu,  and they will appear in the\n" ,
   "order which AFNI chooses.  Otherwise, the plugin menu buttons will\n" ,
   "be alphabetized by default. Alphabetizing is done without regard to\n" ,
   "case (using the C library routine strcasecmp).\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_VOLREG_EDGING\n" ,
   "----------------------------\n" ,
   "This variable affects the operation of 3dvolreg, the volume registration\n" ,
   "plugin, and the 3D registration code in the realtime acquisition plugin.\n" ,
   "It determines the size of the region around the edges of the base volume\n" ,
   "where the default weight will be set to zero.  Call the value of this\n" ,
   "variable 'ee'.  If 'ee' is a plain number (e.g., 5), then it is a voxel\n" ,
   "count, giving the thickness along each face of the 3D brick.  If 'ee' is\n" ,
   "of the form '5%', then it is a fraction of of each brick size.  For\n" ,
   "example, '5%' of a 256x256x124 volume means that 13 voxels on each side\n" ,
   "of the xy-axes will get zero weight, and 6 along the z-axis.  '5%' is\n" ,
   "the default value used by the 3D registration routines (in mri_3dalign.c)\n" ,
   "if no other value is specified.\n" ,
   "\n" ,
   "--------------------\n" ,
   "Variable: AFNI_TRACE\n" ,
   "--------------------\n" ,
   "This variable controls the initial setting of the tracing (debugging)\n" ,
   "code when AFNI programs startup.  If it is set to 'y', then tracing\n" ,
   "is turned on and set to the LOW mode (like -trace in AFNI).  If it is\n" ,
   "set to 'Y', then tracing is turned on and set to the HIGH mode (like\n" ,
   "-TRACE in AFNI).  Anything else, and tracing is turned off.\n" ,
   "\n" ,
   "N.B.: You can't set this variable in .afnirc and expect it to have\n" ,
   "      any effect (and why would you want to?), since it is read from\n" ,
   "      the environment BEFORE the .afnirc file is read in.\n" ,
   "\n" ,
   "N.B.: At this moment (26 Jan 2001), only the AFNI program itself is\n" ,
   "      configured for tracing.  As time goes on, the entire AFNI\n" ,
   "      programming library and suite of programs will be revamped for\n" ,
   "      this purpose.  The goal is to make it easier to find bugs, and\n" ,
   "      localize crashes.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_ROTA_ZPAD\n" ,
   "------------------------\n" ,
   "This variable controls the amount of zero-padding used during 3D rotations\n" ,
   "in 3drotate, 3dvolreg, etc.  It provides a default value for the \"-zpad\"\n" ,
   "options of these programs.  If zero-padding is used, then this many voxels\n" ,
   "are padded out on each edge (all 6 faces) of a 3D brick before rotation.\n" ,
   "After the rotation, these perimeter values (whatever they might be) will\n" ,
   "be stripped off.  If \"-zpad\" is used on the command line, it overrides\n" ,
   "this value.  Zero padding during rotation is useful to avoid edge effects,\n" ,
   "the worst of which is the loss of data off the edge of the volume during\n" ,
   "the 4 shearing stages.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_TO3D_ZPAD\n" ,
   "------------------------\n" ,
   "This variable sets the number of slices added on each z-face in datasets\n" ,
   "created by program to3d.  It provides a default value for the \"-zpad\" option\n" ,
   "of that program.  It can be set to an integer, meaning a slice count, or\n" ,
   "a number of millimeters, meaning a minimum distance to pad:\n" ,
   "   setenv AFNI_TO3D_ZPAD 2\n" ,
   "   setenv AFNI_TO3D_ZPAD 16mm\n" ,
   "If \"-zpad\" is used on the to3d command line, it overrides this value.\n" ,
   "If neither is present, no zero padding is used.  Note well that this\n" ,
   "padding is only in the z-direction, unlike that of AFNI_ROTA_ZPAD.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_IMAGE_MINFRAC (editable)\n" ,
   "----------------------------\n" ,
   "This variable sets the minimum size of an image window when it is first\n" ,
   "opened, in terms of a fraction of the overall screen area.  By default,\n" ,
   "this value is set to 0.02; you can override this by (for example)\n" ,
   "   setenv AFNI_IMAGE_MINFRAC 0.05\n" ,
   "If you set this value to 0.0, then there will be no minimum.  This is\n" ,
   "the old behavior, where the initial window size is always 1 screen pixel\n" ,
   "per data pixel, and can lead to image windows that are hard to resize or\n" ,
   "otherwise use (when the dataset is small).  The largest value I recommend\n" ,
   "for AFNI_IMAGE_MINFRAC is 0.1; however, you can set it to as large as 0.9\n" ,
   "if you are completely crazy, but I'm not responsible for the results --\n" ,
   "don't even think of complaining or commenting to me about problems that\n" ,
   "arise if you try this!\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_IMAGE_MAXFRAC\n" ,
   "----------------------------\n" ,
   "This variable sets the maximum size of an image window, as a fraction of\n" ,
   "the width and height of the screen.  The default value is 0.9.  This lets\n" ,
   "you prevent image windows from auto-resizing to be too big when you\n" ,
   "change datasets.  Note that if you have turned on AFNI_ENFORCE_ASPECT, then\n" ,
   "this feature will prevent you from resizing a window to be larger than\n" ,
   "the AFNI_IMAGE_MAXFRAC fraction of the screen dimensions.\n" ,
   "\n" ,
   "-----------------------\n" ,
   "Variable: AFNI_AUTOGZIP (cf. AFNI_COMPRESSOR) (editable)\n" ,
   "-----------------------\n" ,
   "If this YES/NO variable is set to YES, then when AFNI programs write a\n" ,
   "dataset .BRIK file to disk, they will test to see if the data is easily\n" ,
   "compressible (at least 80%).  If so, then the GZIP compression will be\n" ,
   "used.  (For this to work, the gzip program must be in your path.) This\n" ,
   "can be useful if you are dealing with mask datasets, which are usually\n" ,
   "highly compressible, but don't want the overhead of trying to compress\n" ,
   "and decompress arbitrary MRI datasets.\n" ,
   "\n" ,
   "A command line method to carry out compression of datasets that will\n" ,
   "compress well is to use a csh script like the following:\n" ,
   "\n" ,
   "  #!/bin/csh\n" ,
   "  foreach fred ( `find . -name \\*.BRIK -print` )\n" ,
   "    ent16 -%50 < $fred\n" ,
   "    if( $status == 1 ) gzip -1v $fred\n" ,
   "  end\n" ,
   "\n" ,
   "This will only gzip .BRIK files that the program ent16 estimates will\n" ,
   "compress by at least 50%.  Note that ent16's estimate of compression\n" ,
   "may be high or low relative to what gzip actually does.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_DONT_MOVE_MENUS (editable)\n" ,
   "------------------------------\n" ,
   "If this YES/NO variable is set to YES, then the functions that try\n" ,
   "to move popup menus to \"good\" locations on screens will be skipped.\n" ,
   "This seems to be necessary on some Solaris systems, where the menus\n" ,
   "can end up being moved to bizarre locations.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_MINC_DATASETS\n" ,
   "----------------------------\n" ,
   "If this YES/NO variable is not set to NO, then MINC-format files\n" ,
   "with name suffix .mnc will be read into the interactive AFNI\n" ,
   "program at startup, along with standard .HEAD/.BRIK datasets.\n" ,
   "That is, you have to set this variable explicitly to NO if you\n" ,
   "don't want MINC-format files to be automatically recognized by\n" ,
   "the interactive AFNI program.  This variable does not affect\n" ,
   "the ability of command line programs (3dSomething) to read\n" ,
   ".mnc input files.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_MINC_FLOATIZE\n" ,
   "----------------------------\n" ,
   "If this YES/NO variable is set to YES, then when MINC-format files\n" ,
   "are read in as datasets, their values will be scaled to floats.\n" ,
   "Otherwise, their values will be scaled to the same data type as\n" ,
   "stored in the file.  In some cases, the latter behavior is not\n" ,
   "good; for example, if a byte-valued file (intrinsic range 0..255)\n" ,
   "is scaled to the range 0..0.5 in the MINC header, then after\n" ,
   "conversion back to bytes, the resulting AFNI dataset values will\n" ,
   "all be zero.  Setting AFNI_MINC_FLOATIZE to YES will cause the\n" ,
   "scaled values to be stored as floats.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_MINC_SLICESCALE\n" ,
   "------------------------------\n" ,
   "If this YES/NO variable is set to NO, then AFNI will not use the\n" ,
   "image-min and image-max scaling when reading data from MINC files.\n" ,
   "Normally, you want this scaling, since MINC files are scaled separately\n" ,
   "in each slice.  However, if the image-min and image-max values in the\n" ,
   "MINC file are damaged, then you can turn off the scaling this way.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_ANALYZE_SCALE\n" ,
   "----------------------------\n" ,
   "If this YES/NO variable is set to NO, then the \"funused1\" entry\n" ,
   "in the Mayo Analyze .hdr file will not be used as a scale factor\n" ,
   "for the images contained in the corresponding .img file.  Otherwise,\n" ,
   "if funused1 is positive and not equal to 1.0, all the image data\n" ,
   "in the .img file will be scaled by this value.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_ANALYZE_FLOATIZE\n" ,
   "-------------------------------\n" ,
   "If this YES/NO variable is set to YES, then Mayo Analyze files\n" ,
   "will be scaled to floats on input.  Otherwise, they will be read\n" ,
   "in the format in which they are stored in the .img file.  Conversion\n" ,
   "to floats can be useful if the scaling factor is such that the image\n" ,
   "native format can't hold the scaled values; for example, if short\n" ,
   "values in the image range from -1000..1000 and the scale factor\n" ,
   "is 0.0001, then the scaled values range from -0.1..0.1, which would\n" ,
   "be truncated to 0 in the scaled image if it is not \"floatized\".\n" ,
   "(Conversion to floats will only be done to byte, short, and int\n" ,
   "image types.)\n" ,
   "\n" ,
   "---------------------------------\n" ,
   "Variable: AFNI_ANALYZE_ORIGINATOR\n" ,
   "---------------------------------\n" ,
   "If this YES/NO variable is set to YES, then AFNI will attempt\n" ,
   "to read and use the ORIGINATOR field in a Mayo Analyze file\n" ,
   "to set the origin of the pixel space in AFNI.  This origin\n" ,
   "can be used directly by several programs--the main AFNI viewer,\n" ,
   "and all of the 3dxxxx programs, including especially 3dcopy,\n" ,
   "which is the preferred way to convert an Analyze format file\n" ,
   "to an AFNI native file.\n" ,
   "This variable will also force 3dAFNItoANALYZE to write the\n" ,
   "ORIGINATOR field into the output Analyze file based on the\n" ,
   "input AFNI file's origin information.\n" ,
   "The ORIGINATOR field should be compatible with SPM in most\n" ,
   "cases, but please verify this.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_START_SMALL\n" ,
   "--------------------------\n" ,
   "If this YES/NO variable is set to YES, then when AFNI starts, it will\n" ,
   "look for the smallest datasets in the first session, and choose these\n" ,
   "as its starting point.  This can be useful if you also use the layout\n" ,
   "feature to pop open an image window on startup; in that case, if the\n" ,
   "default starting dataset (the first alphabetical) is huge, you won't\n" ,
   "see anything while the program reads all of into memory before displaying\n" ,
   "the first image.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_MENU_COLSIZE\n" ,
   "---------------------------\n" ,
   "This numerical variable sets the maximum number of entries in a popup\n" ,
   "menu column (e.g., like the sub-brick menus for bucket datasets).  The\n" ,
   "default value is 20, but you may want to make this larger (say 40).  When\n" ,
   "you have a menu with a huge number of entries, the menu can become so\n" ,
   "wide that it doesn't fit on the screen.  Letting the columns be longer\n" ,
   "will make the menus be narrower across the screen.\n" ,
   "\n" ,
   "Another way to get a handle on such huge menus is to Button-3 (right)\n" ,
   "click on the label to the left of the menu.  This will popup a one-\n" ,
   "column scrollable list chooser that is equivalent to the menu.  In this\n" ,
   "way, it is still possible to use menus that have hundreds of entries.\n" ,
   "The maximum number of entries shown at one time in a scrollable list\n" ,
   "chooser is given by variable AFNI_chooser_listmax if it exists, otherwise\n" ,
   "by AFNI_MENU_COLSIZE.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_GLOBAL_SESSION\n" ,
   "-----------------------------\n" ,
   "This variable, if it exists, is the name of a directory that contains\n" ,
   "\"global\" datasets - ones that you want to be visible in each \"Switch Underlay\"\n" ,
   "or \"Switch Overlay\" menu.  Pointers to the datasets read from this directory\n" ,
   "will be appended to the dataset list for each directory read from the\n" ,
   "command line.  In the \"Switch\" choosers, these datasets are marked with\n" ,
   "the character 'G' at the right, and they appear last in the list.\n" ,
   "\n" ,
   "It really only makes sense to put +tlrc datasets in the global session\n" ,
   "directory, since only they can be presumed to be aligned with other datasets.\n" ,
   "Also, it is probably best if you make sure each global anatomical dataset\n" ,
   "has itself as the anatomy parent; this can be enforced by issuing the command\n" ,
   "  3drefit -apar SELF *.HEAD\n" ,
   "in the global session directory.\n" ,
   "\n" ,
   "When you Switch Sessions and are viewing a global dataset, it is likely that\n" ,
   "you will NOT be viewing the same dataset after the Switch Session.  You will\n" ,
   "have to then Switch Underlay and/or Switch Overlay to get back to the same\n" ,
   "global dataset(s).\n" ,
   "\n" ,
   "If you start AFNI and there are no datasets in the sessions given on the\n" ,
   "command line, then the directory specified by this variable becomes the\n" ,
   "default session.  If there are no datasets there, either, then AFNI makes\n" ,
   "up a dummy dataset (AFNI cannot operate without at least one dataset).\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_DISP_SCROLLBARS (editable)\n" ,
   "------------------------------\n" ,
   "If this YES/NO variable is set to YES, then the 'Disp' control window\n" ,
   "(on the image viewers) will have scrollbars attached.  This window has\n" ,
   "grown larger over the years, and for some people with pitifully small\n" ,
   "displays (e.g., laptops), it is now taller than their screens.  If\n" ,
   "activated, this option will prevent the Disp window from being so tall\n" ,
   "and will attach scrollbars so you can access all of its contents.\n" ,
   "\n" ,
   "Note: If you change this value interactively, via Edit Environment, the\n" ,
   "change will only affect Disp windows opened after you 'Set' the variable.\n" ,
   "That is, already opened Disp windows won't suddenly get scrollbars if\n" ,
   "you change this to YES.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_GRAPH_TEXTLIMIT (editable)\n" ,
   "------------------------------\n" ,
   "This numerical variable sets the upper limit on the number of rows shown\n" ,
   "in the Button-3 popup in a sub-graph.  If the number of rows in the popup\n" ,
   "would be more than this value, a text window with scrollbars is used\n" ,
   "instead of a \"spring-loaded\" menu pane.  If you set this value to 1, then\n" ,
   "the text window will always be used.  Note that a text window does not\n" ,
   "automatically popdown, but must be explicitly dismissed by the user\n" ,
   "pressing the \"Quit\" button.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_GRAPH_BASELINE\n" ,
   "-----------------------------\n" ,
   "This variable should be set to one of the strings \"Individual\", \"Common\",\n" ,
   "or \"Global\", corresponding to the choices on the Opt->Baseline menu in\n" ,
   "a graph window.  (Actually, only the first letter of the string will be\n" ,
   "used.)  This variable will determine the initial setting of the Baseline\n" ,
   "menu when a graph window opens.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_GRAPH_GLOBALBASE\n" ,
   "-------------------------------\n" ,
   "Normally, the global baseline for a graph window is set to the smallest\n" ,
   "value found in the entire 3D+time dataset.  This variable lets you specify\n" ,
   "a numerical value to be used for this purpose instead.  Probably the most\n" ,
   "common setting (for those who want to use this feature at all, which means\n" ,
   "Mike Beauchamp) would be\n" ,
   "  setenv AFNI_GRAPH_GLOBALBASE 0\n" ,
   "Of course, you can always change the global baseline from the Opt->Baseline\n" ,
   "menu.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_VALUE_LABEL (editable)\n" ,
   "--------------------------\n" ,
   "If this YES/NO variable is set to YES, then the data value label on the\n" ,
   "Define Overlay control panel will be turned on when only 1 or 2 image\n" ,
   "viewer windows are open.  This will consume more CPU time and redrawing\n" ,
   "time than the default, which is that this label is only turned on when\n" ,
   "all 3 image viewer windows are open.  If you are operating X11 remotely\n" ,
   "over a slow connection, this option should not be turned on.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_SUMA_BOXCOLOR (editable)\n" ,
   "----------------------------\n" ,
   "This string defines the color used for overlaying surface nodes transmitted\n" ,
   "from SUMA to AFNI.  The default is an orangish-yellow.  If you like yellow,\n" ,
   "then do\n" ,
   "   setenv AFNI_SUMA_COLOR yellow\n" ,
   "If this is set to \"none\", then these lines won't be plotted.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_SUMA_LINECOLOR (editable)\n" ,
   "-----------------------------\n" ,
   "This string defines the color used for overlaying the intersection of SUMA\n" ,
   "surface triangles with image slice planes.  The default is white.  If this\n" ,
   "is set to \"none\", then these lines won't be plotted.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_SUMA_BOXSIZE (editable)\n" ,
   "---------------------------\n" ,
   "This variable defines the size of the boxes drawn at each surface node\n" ,
   "transmitted from SUMA.  The default is 0.25, which means that each box is\n" ,
   "plus and minus 1/4 of a voxel size about the node location. If you want a\n" ,
   "larger box, you could try\n" ,
   "   setenv AFNI_SUMA_BOXSIZE 0.5\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_SUMA_LINESIZE (editable)\n" ,
   "----------------------------\n" ,
   "This variable sets the thickness of the lines used when drawing a surface\n" ,
   "intersection overlay.  The units are the width of the entire image;\n" ,
   "reasonable values are in the range 0..0.01; 0 means to draw the thinnest\n" ,
   "line possible.  Since this is editable, you can experiment with it to\n" ,
   "see what looks good.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_NIML_START\n" ,
   "-------------------------\n" ,
   "If this YES/NO variable is set to YES, then NIML listening will be engaged\n" ,
   "from when AFNI starts.  You can also enable NIML from the command line with\n" ,
   "the option \"-niml\", and from the Datamode->Misc menu item \"Start NIML\".\n" ,
   "\n" ,
   "NIML is the mechanism by which AFNI talks to other programs - it is the\n" ,
   "successor to plugouts.  At this moment (Mar 2002), the only external NIML\n" ,
   "program is SUMA - the surface mapper.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_KEEP_PANNING (editable)\n" ,
   "---------------------------\n" ,
   "If this YES/NO variable is set to YES, then when the Zoom pan gets turned\n" ,
   "on in the AFNI image viewer, it will stay on until it is explicitly turned\n" ,
   "off.  (The default behavior is to turn panning off after the user releases\n" ,
   "the mouse button.)\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_IMAGE_LABEL_MODE\n" ,
   "-------------------------------\n" ,
   "This integer determines the placement of the image coordinate labels drawn\n" ,
   "in the AFNI image viewer windows.  The possible values are\n" ,
   "   0  =  Labels are off\n" ,
   "   1  =  Labels in upper left\n" ,
   "   2  =  Labels in upper right\n" ,
   "   3  =  Labels in lower left\n" ,
   "   4  =  Labels in lower right\n" ,
   "   5  =  Labels in upper middle\n" ,
   "   6  =  Labels in lower middle\n" ,
   "You can also control the placement and size of the labels from the Button-3\n" ,
   "(right-click) popup menu attached to the intensity bar to the right of the\n" ,
   "image sub-window.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_IMAGE_LABEL_SIZE\n" ,
   "-------------------------------\n" ,
   "This integer determines the size of the image coordinate labels:\n" ,
   "   0  =  Small\n" ,
   "   1  =  Medium\n" ,
   "   2  =  Large\n" ,
   "   3  =  Huge\n" ,
   "   4  =  Enormous\n" ,
   "\n" ,
   "--------------------------------\n" ,
   "Variable: AFNI_IMAGE_LABEL_COLOR (editable)\n" ,
   "--------------------------------\n" ,
   "This variable controls the color of the image coordinate labels.\n" ,
   "\n" ,
   "----------------------------------\n" ,
   "Variable: AFNI_IMAGE_LABEL_SETBACK (editable)\n" ,
   "----------------------------------\n" ,
   "This variable, a floating point value between 0 and 0.1, determines how\n" ,
   "far from the edge an image coordinate label will be drawn.  The units are\n" ,
   "fractions of the image width/height.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_CROSSHAIR_LINES (editable)\n" ,
   "------------------------------\n" ,
   "If this YES/NO variable is set to YES, then the image crosshairs will be\n" ,
   "drawn using lines rather than pixels.  By default (this is the original\n" ,
   "AFNI way), crosshair lines are drawn the same way as functional overlays:\n" ,
   "by putting color pixels on top of the image.  The new way draws lines on\n" ,
   "top of the image instead.  The difference is quite visible when the image\n" ,
   "is zoomed; overlay by pixels shows the crosshair lines as fat blobs, but\n" ,
   "the lines are drawn as thin as possible, no matter what the image window\n" ,
   "size and zoom factor.\n" ,
   "\n" ,
   "Good points about crosshairs drawn with lines:\n" ,
   " - They are less obtrusive than pixel overlay, especially if you zoom\n" ,
   "     or enlarge the image a lot\n" ,
   " - When doing a montage with Spacing=1, they'll look better in the\n" ,
   "     orthogonal slices.\n" ,
   "Good points about crosshairs drawn with pixel overlay:\n" ,
   " - Pixel overlays can be rendered as translucent (on X11 TrueColor displays);\n" ,
   "     geometrical overlays are always solid color.\n" ,
   "So you have to decide what you need most.  You can change this item using\n" ,
   "the \"Edit Environment\" pseudo-plugin on the Datamode->Misc menu, so you\n" ,
   "can play with it interactively to get the features you want.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_CROP_ZOOMSAVE (editable)\n" ,
   "----------------------------\n" ,
   "When saving a zoomed image, the default is to save the entire zoomed image,\n" ,
   "not just the part you see.  If this YES/NO variable is set to YES, then only\n" ,
   "the visible part will be saved.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variables: AFNI_TLRC_BBOX_*\n" ,
   "---------------------------\n" ,
   "These variables let you choose the size of the \"Talairach Box\", into which\n" ,
   "+tlrc datasets are transformed.  If defined, they should be positive values,\n" ,
   "in mm.  The 5 variables (any or all of which may be used) are:\n" ,
   "\n" ,
   "  AFNI_TLRC_BBOX_LAT = distance from midline to maximum left/right position\n" ,
   "                        [default=80]\n" ,
   "  AFNI_TLRC_BBOX_ANT = distance from AC to most anterior point in box\n" ,
   "                        [default=80]\n" ,
   "  AFNI_TLRC_BBOX_POS = distance from AC to most posterior point in box\n" ,
   "                        [default=110]\n" ,
   "  AFNI_TLRC_BBOX_INF = distance from AC-PC line to most inferior point in box\n" ,
   "                        [default=55 for small box, 65 for big box]\n" ,
   "  AFNI_TLRC_BBOX_SUP = distance from AC-PC line to most superior point in box\n" ,
   "                        [default=85]\n" ,
   "\n" ,
   "For example, \"setenv AFNI_TLRC_BBOX_INF 100\" lets you define the +tlrc box\n" ,
   "to extend 100 mm below the AC-PC line.  Please note that virtually all the\n" ,
   "3d* analysis programs (3dANOVA, etc.) do voxel-by-voxel analyses.  This fact\n" ,
   "means that you will be unable to compare datasets created in +tlrc coordinates\n" ,
   "with different box sizes.  Also, you will be unable to overlay regions from\n" ,
   "the Talairach Daemon database onto odd-sized +tlrc datasets.  Therefore, I\n" ,
   "recommend that these variables be used only when strictly needed, and with\n" ,
   "caution.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_TTRR_SETUP\n" ,
   "-------------------------\n" ,
   "Name of a file to be loaded to define Talairach Atlas Colors, when the Atlas\n" ,
   "Colors control panel is first created.  Format is the same as a file created\n" ,
   "from this control panel's \"Save\" button.  This filename should be an absolute\n" ,
   "path (e.g., /home/yourname/.afni_ttcolors), since otherwise it will be read\n" ,
   "relative to the directory in which you start AFNI.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_LOAD_PRINTSIZE\n" ,
   "-----------------------------\n" ,
   "AFNI will print (to stderr) a warning that it is loading a large dataset from\n" ,
   "disk.  This value determines the meaning of \"large\".  For example, setting\n" ,
   "this variable to \"40M\" means that loading a dataset larger than 40 Megabytes\n" ,
   "will trigger the warning.  If not given, the default value is 100 Megabytes.\n" ,
   "The purpose of the warning is just to let the user know that it may be several\n" ,
   "seconds before the dataset is loaded (e.g., before the images appear).  If you\n" ,
   "don't want this warning at all, set this variable to the string \"0\".\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_ANALYZE_DATASETS\n" ,
   "-------------------------------\n" ,
   "If this YES/NO variable is not set to NO, then ANALYZE-format files with name\n" ,
   "suffix .hdr will be read into the interactive AFNI program at startup, along\n" ,
   "with standard .HEAD/.BRIK datasets.  That is, you have to set this variable\n" ,
   "explicitly to NO if you don't want ANALYZE-format files to be automatically\n" ,
   "recognized by the interactive AFNI program.  This variable does not affect\n" ,
   "the ability of command line programs (3dSomething) to read .hdr input files.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_ANALYZE_ORIENT\n" ,
   "-----------------------------\n" ,
   "ANALYZE .hdr files do not contain reliable information about the orientation\n" ,
   "of the data volumes.  By default, AFNI assumes that these datasets are\n" ,
   "oriented in LPI order.  You can set this variable to a different default\n" ,
   "order.  See AFNI_ORIENT for details on the 3 letter format for this.\n" ,
   "\n" ,
   "---------------------------------\n" ,
   "Variable: AFNI_ANALYZE_AUTOCENTER\n" ,
   "---------------------------------\n" ,
   "ANALYZE .hdr files do not contain information about the origin of coordinates.\n" ,
   "The default AFNI approach mirrors that of FSL - the outermost corner of the\n" ,
   "first voxel in the dataset is set to (0,0,0).  If you set this variable\n" ,
   "(AFNI_ANALYZE_AUTOCENTER) to YES, then instead (0,0,0) will be set to the\n" ,
   "center of the 3D ANALYZE array.  This is the default that would be applied\n" ,
   "if you read the ANALYZE array into program to3d.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_VERSION_CHECK\n" ,
   "----------------------------\n" ,
   "If this YES/NO variable is set to NO, then AFNI will not try to check if its\n" ,
   "version is up-to-date when it starts.  Otherwise, it will try to check the\n" ,
   "program version with the AFNI web server.\n" ,
   "\n" ,
   "-----------------------------------\n" ,
   "Variable: AFNI_SLICE_SPACING_IS_GAP\n" ,
   "-----------------------------------\n" ,
   "This YES/NO variable is designed to patch a flaw in some DICOM files, where\n" ,
   "the \"Spacing Between Slices\" attribute is erroneously set to the gap between\n" ,
   "the slices, rather than the center-to-center slice distance specified in\n" ,
   "the DICOM standard.  If this variable is set to YES, then the \"Slice Thickness\"\n" ,
   "attribute will always be added to \"Spacing Between Slices\" to get the z voxel\n" ,
   "size (assuming both attributes are present in the DICOM file).\n" ,
   "\n" ,
   "To check if a DICOM file has this problem, you can read it into to3d with\n" ,
   "the command \"to3d suspect_file_name\".  A warning will be printed to the\n" ,
   "terminal window if attribute \"Spacing Between Slices\" is less than\n" ,
   "attribute \"Slice Thickness\".  Another way to check is with a command like so\n" ,
   "\n" ,
   "  dicom_hdr suspect_file_name | grep \"Slice\"\n" ,
   "\n" ,
   "then check if the \"Spacing Between Slices\" and \"Slice Thickness\" values are\n" ,
   "correct for the given acquisition.  We have only seen this problem in GE\n" ,
   "generated DICOM files, but that doesn't mean it won't occur elsewhere.\n" ,
   "\n" ,
   "If this variable is set to NO, then this patchup will never be made.\n" ,
   "The z voxel size will be set to \"Spacing Between Slices\" if present,\n" ,
   "otherwise to \"Slice Thickness\".  This may be needed for some Phillips pulse\n" ,
   "sequences, which can report \"Spacing Between Slices\" < \"Slice Thickness\".\n" ,
   "In such a case, if this variable is not set, the wrong z voxel size will\n" ,
   "be assigned!\n" ,
   "\n" ,
   "If this variable is not set at all, AND if \"Spacing Between Slices\" is less\n" ,
   "less than 0.99 times \"Slice Thickness\", it will be treated as a gap;\n" ,
   "that is, the z voxel size will again be set to \"Spacing Between Slices\" +\n" ,
   "\"Slice Thickness\" if \"Spacing Between Slices\" < 0.99*\"Slice Thickness\".\n" ,
   "Otherwise, the z voxel size will be set to the larger of\n" ,
   "\"Spacing Between Slices\" and \"Slice Thickness\".\n" ,
   "\n" ,
   "N.B.: \"YES\", \"NO\", and \"not set\" have 3 different sets of behavior!\n" ,
   "      In the summary below, if a variable isn't set, treat it as zero:\n" ,
   "\n" ,
   "  YES     => dz = Thickness + Spacing\n" ,
   "  NO      => dz = Spacing if present, otherwise Thickness\n" ,
   "  not set => if( Spacing > 0 && Spacing < 0.99*Thickness )\n" ,
   "               dz = Thickness + Spacing\n" ,
   "             else\n" ,
   "               dz = MAX( Thickness , Spacing )\n" ,
   "\n" ,
   "If neither variable is set, then dz=1 mm, which is probably wrong.\n" ,
   "\n" ,
   "Sorry about this complexity, but the situation with various manufacturers\n" ,
   "is complicated, murky, and confusingly maddening.\n" ,
   "\n" ,
   "---------------------------------------------------\n" ,
   "Variables: AFNI_DICOM_RESCALE and AFNI_DICOM_WINDOW\n" ,
   "---------------------------------------------------\n" ,
   "DICOM image files can contain rescaling and windowing \"tags\".  If present,\n" ,
   "these values indicate to affinely modify the values stored in the file.\n" ,
   "As far as I can tell, \"rescale\" means that the values should always be\n" ,
   "modified, whereas \"window\" means the values should be modified for display\n" ,
   "purposes.  If both are present, the rescale comes before window.  These\n" ,
   "two YES/NO environment variables control whether the AFNI image input\n" ,
   "functions (used in to3d) should apply the rescale and window tags.\n" ,
   "\n" ,
   "It is my impression from the laconic, terse, and opaque DICOM manual that\n" ,
   "window tags are intended for display purposes only, and that they aren't\n" ,
   "needed for signal processing.  But you'll have to examine your own data to\n" ,
   "decide whether to use these options -- manufacturers seem to differ.\n" ,
   "Plus, I don't have that much experience with DICOM data from many different\n" ,
   "sources.\n" ,
   "\n" ,
   "-----------------------\n" ,
   "Variable: IDCODE_PREFIX\n" ,
   "-----------------------\n" ,
   "AFNI stores with each dataset a unique string, called an \"idcode\".  An example\n" ,
   "is \"XYZ_MoNLqdNOwMNEYmKSBytfJg\".  You can alter the first three characters of\n" ,
   "the idcode with this variable.  For example,\n" ,
   "  setenv IDCODE_PREFIX RWC\n" ,
   "sets the first 3 characters of newly generated idcodes to be the initials of\n" ,
   "AFNI's author.  I find this a handy way to \"brand\" my datasets.  Of course,\n" ,
   "there are only 17576 possible 3 letter combinations (140608 if you allow for\n" ,
   "case), so you should claim your prefix soon!!!\n" ,
   "\n" ,
   "Idcodes are used to store links between datasets.  For example, when SUMA\n" ,
   "sends a surface to AFNI, it identifies the dataset to which the surface is\n" ,
   "to be associated with the dataset's idcode.  Similarly, when AFNI sends a\n" ,
   "color overlay to SUMA, it uses the surface idcode to indicate which surface\n" ,
   "family the overlay is to be mapped onto.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_AGIF_DELAY\n" ,
   "-------------------------\n" ,
   "This is the time delay between frames when writing an animated GIF file from\n" ,
   "an image viewer window.  The units are 100ths of seconds; the default value\n" ,
   "is 20 (= 5 frames per second).\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_MPEG_FRAMERATE\n" ,
   "-----------------------------\n" ,
   "This value sets the frame rate (per second) of the MPEG-1 output animation\n" ,
   "from the image viewer window.  The legal values allowed by MPEG-1 are\n" ,
   "24, 25, 30, 50, and 60; 24 is the default.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_STARTUP_SCRIPT\n" ,
   "-----------------------------\n" ,
   "If this is set, this is the name of an AFNI Script to run when AFNI first\n" ,
   "starts.  (See the file README.driver for information about AFNI Scripts.)\n" ,
   "If this is not set, it defaults to \".afni.startup_script\".  The program first\n" ,
   "tries to read this filename from the current working directory; if that fails,\n" ,
   "then it tries to read from your home directory.  No error message is given\n" ,
   "if neither file can be read.\n" ,
   "\n" ,
   "You can save a file \".afni.startup_script\" that will recreate the window\n" ,
   "layout you currently have.  Use the \"Datamode->Misc->Save Layout\" button\n" ,
   "and press \"Set\" on the popup control without entering any filename.  Instead\n" ,
   "of a Layout file (cf. AFNI_LAYOUT_FILE above), you'll get a Script file.\n" ,
   "\n" ,
   "The capabilities of Script files are being expanded.  Not all features of\n" ,
   "the AFNI window setup are currently save-able this way.\n" ,
   "\n" ,
   "You can load a Script file interactively during an AFNI run by using the\n" ,
   "button \"Datamode->Misc->Run Script\".  As a 'secret' option, if you enter\n" ,
   "a line containing a blank in the the filename dialog, that line will be\n" ,
   "executed as a single command, rather than be used as a script filename.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_DEFAULT_OPACITY\n" ,
   "------------------------------\n" ,
   "This should be set to an integer from 1..9, and controls the default opacity\n" ,
   "setting for the color overlay in image viewer windows.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_DEFAULT_IMSAVE\n" ,
   "-----------------------------\n" ,
   "This should be set to the suffix of the image format to which you want to save\n" ,
   "from an image viewer.  The suffixes AFNI knows about (as of 23 Jan 2003) are\n" ,
   " ppm = Portable PixMap format                            = cat\n" ,
   " jpg = Joint Photographics Experts Group (JPEG) format   = cjpeg\n" ,
   " gif = Compuserve Graphics Interchange File (GIF) format = ppmtogif\n" ,
   " tif = Tagged Image File Format (TIFF)                   = ppm2tiff or pnmtotiff\n" ,
   " bmp = Windows Bitmap (BMP) format                       = ppmtobmp\n" ,
   " eps = Encapsulated PostScript format                    = pnmtops\n" ,
   " pdf = Portable Document Format                          = epstopdf\n" ,
   " png = Portable Network Graphics format                  = pnmtopng\n" ,
   "The third column is the name of the external filter program that AFNI uses to\n" ,
   "write the format.  If a filter is not present on your system, then that option\n" ,
   "is not available.  Most of these filters are part of the netpbm package.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variables: AFNI_COLORSCALE_xx  for xx=01, 02, ..., 99\n" ,
   "-----------------------------\n" ,
   "These variables let you name files to be read it at AFNI startup to define\n" ,
   "\"continuous\" colorscales for the \"**\" mode of the color pbar.  These files\n" ,
   "will be looked for in the current directory when you start AFNI, or in your\n" ,
   "home directory (if they aren't in the current directory).  A sample file:\n" ,
   "\n" ,
   "  Yellow-Red-Blue\n" ,
   "  1.0 #ffff00\n" ,
   "  0.7 #ffaa00\n" ,
   "  0.5 #ff0000\n" ,
   "  0.3 #aa00aa\n" ,
   "  0.0 #0000ff\n" ,
   "\n" ,
   "The first line is the name of this colorscale, to go in the colorscale popup\n" ,
   "chooser.  The succeeding lines each have a number and a color definition.\n" ,
   "The numbers should be decreasing, and indicate the location on the colorscale.\n" ,
   "The largest number corresponds to the top of the colorscale and the smallest\n" ,
   "to the bottom - intermediate numbers denote intermediate locations.  The colors\n" ,
   "at each location are specified using X11 notation (cf. \"man XParseColor\").\n" ,
   "In this example, I'm using hexadecimal colors, in the form #rrggbb, where each\n" ,
   "hex pair ranges from 00 to ff.  Another color format is \"rgbi:rf/gf/bf\",\n" ,
   "where each value rf,gf,bf is a number between 0.0 and 1.0 (inclusive); for\n" ,
   "example, yellow would be \"rgbi:1.0/1.0/0.0\".\n" ,
   "\n" ,
   "Colors are interpolated (linearly in RGB space) between the break locations\n" ,
   "given in the file.  There are actually 128 color locations on a colorscale.\n" ,
   "\n" ,
   "An alternative format for the file is to omit the numbers indicating the\n" ,
   "break locations.  In this case, the break locations will be taken to be\n" ,
   "equally spaced.  For example:\n" ,
   "\n" ,
   "  Yellow-Red-Blue\n" ,
   "   #ffff00\n" ,
   "   #ffaa00\n" ,
   "   #ff0000\n" ,
   "   #aa00aa\n" ,
   "   #0000ff\n" ,
   "\n" ,
   "This example is not exactly the same as the other one, since the breakpoints\n" ,
   "are evenly spaced now (as if they had been given as 1.0, 0.75, 0.5, 0.25,\n" ,
   "and 0.0).  With this format, if you want to manually specify all 128 colors,\n" ,
   "you can do so, 1 color per line, remembering that the first line of the file\n" ,
   "is taken to be the colorscale title (no blanks allowed in the title!).\n" ,
   "\n" ,
   "---------------------------------\n" ,
   "Variable: AFNI_COLORSCALE_DEFAULT\n" ,
   "---------------------------------\n" ,
   "If set, this is the name of the default colorscale to use in setup.  As a\n" ,
   "special case, if you DO NOT want a colorscale to be setup by default at all,\n" ,
   "then set this variable to the string \"NO\".\n" ,
   "N.B.: This variable only applies if you are using AFNI with a TrueColor X11\n" ,
   "visual.  If you are using a PseudoColor visual, then this variable is ignored!\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_RESCAN_METHOD\n" ,
   "----------------------------\n" ,
   "On 28 Dec 2002, I modified the way that the \"Rescan\" operation in AFNI works\n" ,
   "when re-reading datasets from sessions.  The old way would purge and replace\n" ,
   "all datasets; the new way just adds datasets that didn't exist before.\n" ,
   "There are some differences between these methods:\n" ,
   "  \"Replace\" will detect changes to a dataset, so if you add a brick using\n" ,
   "    3dTcat -glueto (for example), this will be reflected in AFNI.\n" ,
   "  \"Replace\" will cause troubles if you are using a dataset in a plugin;\n" ,
   "    the two main examples are volume rendering and the drawing plugin.\n" ,
   "    This problem will occur even if you didn't do anything to the dataset\n" ,
   "    on disk, since the internal pointer to the dataset will have been\n" ,
   "    changed by the rescan, but the plugins won't know that.\n" ,
   "  \"Add\" will not detect changes to a dataset on disk, but it also won't\n" ,
   "    affect the pointers to the existing datasets.\n" ,
   "You can choose to use the \"Replace\" method (the old style) by setting\n" ,
   "this environment variable to the string \"REPLACE\".\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_OLD_PPMTOBMP\n" ,
   "---------------------------\n" ,
   "The old (before 21 Feb 2003) usage of netpbm program \"ppmtobmp\" was to\n" ,
   "write a color image quantized to 255 colors.  The new usage is to write\n" ,
   "a 24-bit image, which is thus not color-quantized.  If you want the old\n" ,
   "behavior, set this environment variable to YES.  This setting (YES) will\n" ,
   "be necessary if you have an older version of ppmtobmp in your path, which\n" ,
   "doesn't support the \"-bpp\" option.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_1DPLOT_COLOR_xx\n" ,
   "------------------------------\n" ,
   "This variable lets you set the colors used in the 1dplot program (and other\n" ,
   "similar graphs).  Here, \"xx\" is a number from \"01\" to \"19\".  The value of\n" ,
   "the environment variable must be in the form \"rgbi:rf/gf/bf\", where each\n" ,
   "color intensity (rf, gf, bf) is a number between 0.0 and 1.0.  For example,\n" ,
   "\"rgbi:1.0/1.0/0.0\" is yellow.  By default, the first 4 colors are defined\n" ,
   "as the equivalents of\n" ,
   "  setenv AFNI_1DPLOT_COLOR_01 rgbi:0.0/0.0/0.0\n" ,
   "  setenv AFNI_1DPLOT_COLOR_02 rgbi:0.9/0.0/0.0\n" ,
   "  setenv AFNI_1DPLOT_COLOR_03 rgbi:0.0/0.7/0.0\n" ,
   "  setenv AFNI_1DPLOT_COLOR_04 rgbi:0.0/0.0/0.9\n" ,
   "which are black, red, green, and blue, respectively.  You can alter these\n" ,
   "colors, or leave them unchanged and start defining colors at 05.  The largest\n" ,
   "color number you define will be the last color index used; if more line colors\n" ,
   "are needed, they will cycle back to color 01.  If you leave a gap in the\n" ,
   "numbering (e.g., you define color 07 but not 05 or 06), then the undefined\n" ,
   "colors will be black.\n" ,
   "\n" ,
   "---------------------------------\n" ,
   "Variable: AFNI_SIEMENS_INTERLEAVE\n" ,
   "---------------------------------\n" ,
   "The old (pre-DICOM) Siemens .ima image mosaic format sometimes stores the\n" ,
   "multi-slice EPI data in correct spatial order and sometimes in correct\n" ,
   "time acquisition order.  In the latter case, the images are stored in\n" ,
   "a spatially-interleaved fashion.  As far as I know, there is no way to\n" ,
   "tell this from the .ima file header itself.  Therefore, if you have a\n" ,
   "problem with such files, set this variable to YES to un-interleave the\n" ,
   "images when to3d reads them.  One way to tell if the images need to be\n" ,
   "un-interleaved is to do\n" ,
   "  afni -im fred.ima\n" ,
   "then look at the images in an Axial image viewer.  If the slices make up\n" ,
   "a single coherent volume, then they are NOT interleaved.  If the slices\n" ,
   "look like they make up 2 separate brain volumes, then they need to be\n" ,
   "un-interleaved, and you need to set this variable to YES.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_TRY_DICOM_LAST\n" ,
   "-----------------------------\n" ,
   "When to3d tries to read an image file, it guesses the format from the\n" ,
   "filename.  However, this doesn't always work.  In particular, DICOM files\n" ,
   "don't have any fixed filename suffix or prefix.  If all else fails, to3d\n" ,
   "normally tries to read a file as a DICOM file, and as a last resort, as\n" ,
   "a flat binary file.  However, if a file is NOT a DICOM file, the DICOM\n" ,
   "reading function will print out a lot of error messages, since there is\n" ,
   "also no standard internal marker in all DICOM files that identify them.\n" ,
   "Most people don't like all these messages (perhaps hundreds per file),\n" ,
   "even if the program then successfully reads their flat binary files.\n" ,
   "\n" ,
   "If this YES/NO variable is set to YES, then the normal last-resort order\n" ,
   "of reading described above is reversed.  If to3d can't read the file any\n" ,
   "other way, it will try it as a flat binary file.  If that fails, then\n" ,
   "DICOM will be the ultimate resort, instead of being the penultimate\n" ,
   "resort that it is by default.  This may help elide some error messages.\n" ,
   "However, if you have a DICOM file that is exactly 131072 bytes long\n" ,
   "(for example), then it will be interpreted as a headerless 256x256 image\n" ,
   "of shorts, instead of whatever it really is.  So only set this variable\n" ,
   "to YES if necessary!\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_THRESH_BIGSTEP\n" ,
   "-----------------------------\n" ,
   "The AFNI threshold sliders (in the Define Overlay control panels and the\n" ,
   "Render Dataset plugins) are divided into 10000 steps from bottom to top.\n" ,
   "If you click in the trough or use the PageUp/PageDown keys, the default\n" ,
   "action is to move the slider 10 of the steps at once.  (The up and down\n" ,
   "arrow keys move 1 step at a time.)  You can change this big step from the\n" ,
   "default of 10 to any value between 1 and 1000 by setting this environment\n" ,
   "variable; for example\n" ,
   "  setenv AFNI_THRESH_BIGSTEP 100\n" ,
   "will move the slider 1% of its height per PageUp/PageDown key or mouse click.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_SNAPFILE_PREFIX\n" ,
   "------------------------------\n" ,
   "Image files saved with the \"snapfile\" (or \"record to file\") by default have\n" ,
   "filenames of the form \"S_000001.ppm\".  The prefix \"S\" can be altered by\n" ,
   "setting this environment variable; for example,\n" ,
   "  setenv AFNI_SNAPFILE_PREFIX Elvis\n" ,
   "will save snapfiles with names like \"Elvis_000666.ppm\".  You can view snapfiles\n" ,
   "with the \"aiv\" (\"AFNI Image Viewer\") utility, the \"xv\" program, or many other\n" ,
   "Unix utilities.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_STARTUP_WARNINGS\n" ,
   "-------------------------------\n" ,
   "When the interactive AFNI program starts, it may pop up warnings about the\n" ,
   "programming environment for which it was compiled.  At this time, there are\n" ,
   "two such warning messages possible:\n" ,
   "  LessTiff: AFNI will work with LessTif, but works better with Motif.\n" ,
   "  Button-3: On Solaris 2.8, Button-3 popup menus don't work quite properly.\n" ,
   "If you are tired of seeing these messages, set AFNI_STARTUP_WARNINGS to NO.\n" ,
   "\n" ,
   "----------------------\n" ,
   "Variable: AFNI_1D_TIME\n" ,
   "----------------------\n" ,
   "If this YES/NO variable is set to YES, then when a multicolumn .1D file is\n" ,
   "read in as an AFNI dataset, the column variable is taken to be time, and\n" ,
   "a time-dependent dataset is created.  The default is to create a bucket\n" ,
   "dataset.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_1D_TIME_TR\n" ,
   "-------------------------\n" ,
   "If this is set, and AFNI_1D_TIME is YES, then this determines the TR (in\n" ,
   "seconds) of a .1D file read in as an AFNI dataset.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_3D_BINARY\n" ,
   "------------------------\n" ,
   "If this is set to YES, then .3D files are written by AFNI programs in\n" ,
   "binary, rather than the default text mode.  Binary files will be more\n" ,
   "compact (usually) and faster to read in.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_MAX_OPTMENU (editable)\n" ,
   "--------------------------\n" ,
   "This variable (default=255) sets the maximum number of entries allowed\n" ,
   "in an AFNI \"option menu\" -- these are the buttons that popup a menu\n" ,
   "of values from which to choose, and which also let you popup a text\n" ,
   "list chooser by right-clicking in the menu's label.  (Example: the\n" ,
   "sub-brick option menus \"Anat\", \"Func\", \"Thr\" on the \"Define Overlay\"\n" ,
   "control panel.)\n" ,
   "\n" ,
   "Some computer systems may crash when an option menu gets too big.\n" ,
   "That's why there is a default limit in AFNI of 255 entries.  However,\n" ,
   "if you have a bucket dataset with more than 255 sub-bricks, this makes\n" ,
   "it impossible to view the later data volumes.  If this problem arises,\n" ,
   "you can try setting this environment variable to a larger limit (e.g.,\n" ,
   "99999 would take care of all currently imaginable cases).\n" ,
   "\n" ,
   "---------------------------------\n" ,
   "Variable: AFNI_VALUE_LABEL_DTABLE\n" ,
   "---------------------------------\n" ,
   "This variable sets a filename that holds a default value-label table\n" ,
   "for the Draw Dataset plugin.  A sample file is shown below:\n" ,
   "\n" ,
   "   <VALUE_LABEL_DTABLE\n" ,
   "     ni_type=\"2*String\"\n" ,
   "     ni_dimen=\"3\" >\n" ,
   "    \"1\" \"elvis\"\n" ,
   "    \"2\" \"presley\"\n" ,
   "    \"3\" \"memphis\"\n" ,
   "   </VALUE_LABEL_DTABLE>\n" ,
   "\n" ,
   "The 'ni_dimen' attribute is the number of value-label pairs; in the\n" ,
   "  above example it is 3.\n" ,
   "Each value-label pair is shown on a separate line.  The values and\n" ,
   "  labels are strings, enclosed in quote characters.  There should be\n" ,
   "  exactly as many value-label pairs as specified in 'ni_dimen'.\n" ,
   "If you really want to put a double quote character \" in a label,\n" ,
   "  you can enclose the label in single forward quotes ' instead.\n" ,
   "When you 'Save' a drawn dataset from the Draw Dataset plugin, the\n" ,
   "  .HEAD file attribute VALUE_LABEL_DTABLE will contain a table in\n" ,
   "  exactly this XML-based format.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_STROKE_THRESHOLD (editable)\n" ,
   "-------------------------------\n" ,
   "If you press Button-1 in an image window, and then move it left or\n" ,
   "right (\"stroke it\") before releasing the button, the grayscale mapping\n" ,
   "changes in the same way as if you pressed the 'c' button up and the 'b'\n" ,
   "button down.  This variable sets the threshold for the stroking movement\n" ,
   "size in pixels; a movement of this number of pixels rightwards corresponds\n" ,
   "to one press of 'c' up and 'b' down, while a leftwards movement is like\n" ,
   "one press of 'c' down and 'b' up.  Larger movements make larger adjustments.\n" ,
   "\n" ,
   "A larger threshold makes the stroking less sensitive; a smaller threshold\n" ,
   "makes it more sensitive.  The value you choose will depend on your personal\n" ,
   "taste.  The default is 32 pixels, which is the flavor I prefer.  If you set\n" ,
   "this variable to 0, then the stroking function is disabled.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_STROKE_AUTOPLOT (editable)\n" ,
   "-------------------------------\n" ,
   "If this variable is set to YES, then the graymap-versus-data value plot\n" ,
   "(manually controlled by \"Display Graymap Plot\") is automatically popped\n" ,
   "up when the grayscale mapping is altered by using the stroking feature\n" ,
   "described above.  When the stroke is finished, the plot will pop down.\n" ,
   "N.B.: when the 'Draw Dataset' plugin is active, this option is disabled\n" ,
   "temporarily.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_IMAGE_MINTOMAX (editable)\n" ,
   "-----------------------------\n" ,
   "If this variable is set to YES, then image viewer windows will be set\n" ,
   "to the \"Min-to-Max\" state rather than the default \"2%-to-98%\" state\n" ,
   "when they are opened.  If you set this in the \"Edit Environment\"\n" ,
   "control, it only affects image viewer windows opened after that point.\n" ,
   "\n" ,
   "--------------------------------\n" ,
   "Variable: AFNI_IMAGE_GLOBALRANGE (editable)\n" ,
   "--------------------------------\n" ,
   "If this variable is set to YES, then the image viewer windows will be\n" ,
   "set to scale the bottom gray level to the minimum value in the 3D\n" ,
   "volume and the top gray level to the maximum value in the 3D volume.\n" ,
   "This setting overrides the \"Min-to-Max\" and \"2%-to-98%\" settings in\n" ,
   "the \"Disp\" control panel.  This setting also applies to all image\n" ,
   "viewers.  If you set this in the \"Edit Environment\" control, it will\n" ,
   "apply to all open image viewers immediately, as well as to any image\n" ,
   "viewers opened later.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_DRAW_UNDOSIZE (editable)\n" ,
   "----------------------------\n" ,
   "This variable sets the size (in units of Megabytes) of the Undo/Redo\n" ,
   "buffer in the Draw Dataset plugin.  The default value is 6.  If you\n" ,
   "are short on memory, you could set this to 1.  If you are running out\n" ,
   "of undo levels, you could set this to a larger value; however, this\n" ,
   "would only be needed if you are drawing huge 3D swaths of data at a\n" ,
   "time (e.g., using the 3D sphere option with a large radius).\n" ,
   "\n" ,
   "---------------------\n" ,
   "Variable: AFNI_SPEECH (editable)\n" ,
   "---------------------\n" ,
   "If this YES/NO variable is set to NO, then the AFNI speech synthesis\n" ,
   "is disabled.  At the current time (Nov 2003), only the Mac OS X 10.3\n" ,
   "version of AFNI uses speech synthesis in any way.  And that's just\n" ,
   "for fun.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_IMAGE_ZEROCOLOR\n" ,
   "------------------------------\n" ,
   "This variable, if set to the string name of one of the colors in the\n" ,
   "color chooser menus (e.g., \"Black\"), will result in voxels whose value\n" ,
   "is 0 being set to this color in the slice viewing windows (except when\n" ,
   "viewing RGB images).  The main function is to avoid having to use the\n" ,
   "\"Choose Zero Color\" menu all the time, especially when you use the \"Swap\"\n" ,
   "feature to invert the grayscale map (e.g., to make a T2 weighted image\n" ,
   "look sort of like a T1 weighted image).\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_MPEG_DATASETS\n" ,
   "----------------------------\n" ,
   "This variable can be used to allow MPEG files to be read in as AFNI datasets.\n" ,
   "Such datasets are inherently 3 dimensional.  How they will be organized inside\n" ,
   "AFNI depends on the setting of this variable.  The options are:\n" ,
   "  SPACE = the frame sequence number will be the z-axis\n" ,
   "  TIME  = the frame sequence number will be the time axis\n" ,
   "  NO    = MPEG files won't be read as AFNI datasets\n" ,
   "          (they can still be read as images into to3d, aiv, etc.)\n" ,
   "If this variable is NOT set to anything, then it is the same as SPACE.\n" ,
   "\n" ,
   "MPEG filenames input to AFNI programs (as sources of images or as datasets)\n" ,
   "must end in \".mpg\", \".MPG\", \".mpeg\", or \".MPEG\".  MPEG datasets will be read\n" ,
   "so that the individal images are displayed in an Axial image window.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_MPEG_GRAYIZE\n" ,
   "---------------------------\n" ,
   "If this YES/NO variable is set to YES, then MPEG files read into AFNI,\n" ,
   "to3d, or aiv will be converted to grayscale, even if the images in\n" ,
   "the movie are in color.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_VIDEO_DELAY (editable)\n" ,
   "--------------------------\n" ,
   "This is the number of milliseconds the AFNI waits between drawing new\n" ,
   "images when the 'V' or 'v' keys are pressed in an image (or graph)\n" ,
   "window.  The default value is 1, which is faster than video can be\n" ,
   "displayed anyway.  Set this to a larger value (e.g, 100) to slow down\n" ,
   "the image redraw rate.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_IMAGE_ENTROPY (editable)\n" ,
   "----------------------------\n" ,
   "If this numeric variable is set, this is the entropy of an image below\n" ,
   "which the 2%-98% image scaling will be disabled, and min-to-max will\n" ,
   "be used instead.  The units are bits/byte; a useful threshold seems to\n" ,
   "be in the range (0.2,0.5).  For images that only have a few values\n" ,
   "different from 0, the 2%-98% scaling can produce weird artifacts.  Such\n" ,
   "images will also have a very low entropy.  Since this variable can be\n" ,
   "changed interactively from the Edit Environment controls, you can play\n" ,
   "with it to see how it affects your images.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_LOGO16 (etc.)\n" ,
   "----------------------------\n" ,
   "If this variable is set to YES, then the 'AFNI' background logo used in\n" ,
   "the controller and image windows will be enabled.  By default, it is off.\n" ,
   "You can control the colors of this logo by the following variables:\n" ,
   "  AFNI_LOGO16_FOREGROUND_x\n" ,
   "  AFNI_LOGO16_BACKGROUND_x\n" ,
   "where 'x' is 'A', 'B', 'C', etc., for the various controller labels.\n" ,
   "If AFNI_LOGO16_BACKGROUND_x isn't set, then AFNI_LOGO16_BACKGROUND\n" ,
   "(with no suffix) is checked as an alternate.  The values of these\n" ,
   "variables should be the names of one of the labels on the color chooser\n" ,
   "menus (e.g., the \"Xhairs Color\" menu).  You can use these variables to\n" ,
   "make the windows for the various controllers somewhat distinct in\n" ,
   "appearance.  If these color variables are not set at all, then AFNI\n" ,
   "uses some colors of my choosing for this purpose.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_THRESH_LOCK (editable)\n" ,
   "--------------------------\n" ,
   "This variable can be used to lock the Define Overlay threshold sliders\n" ,
   "together.  There are three possibilities:\n" ,
   "  NO (the default) => each controller's slider is independent\n" ,
   "  VALUE            => the numerical value on each slider will be the same\n" ,
   "  P-VALUE          => the p-value for each slider will be the same\n" ,
   "This locking only applies to AFNI controllers that are Lock-ed together\n" ,
   "(cf. AFNI_ALWAYS_LOCK and the Define Datamode->Lock menu).  If p-values\n" ,
   "are locked, this lock will also only apply to controllers whose current\n" ,
   "Threshold sub-brick has a known statistical distribution.\n" ,
   "\n" ,
   "When you drag a locked threshold slider, the other one will only change\n" ,
   "when you release the mouse button -- they won't slide in tandem, but will\n" ,
   "just jump to the final value.\n" ,
   "\n" ,
   "------------------------\n" ,
   "Variable: AFNI_PBAR_LOCK (editable)\n" ,
   "------------------------\n" ,
   "If this variable is set to YES, then the Define Overlay color bars\n" ,
   "(the \"pbars\") of AFNI controllers that are Lock-ed together will be\n" ,
   "coordinated.  Changes to one locked pbar will be reflected in the\n" ,
   "others immediately.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_IMAGE_ZOOM_NN (editable)\n" ,
   "----------------------------\n" ,
   "If this variable is set to YES, then image viewer windows will use\n" ,
   "nearest neighbor interpolation for zooming.  The default is linear\n" ,
   "interpolation, which produces smoother-looking images.  However, some\n" ,
   "people want to see the actual data values represented in the window,\n" ,
   "not some fancy-schmancy interpolated values designed to look good but\n" ,
   "in fact making a mockery of a sham of a mockery of a travesty of two\n" ,
   "mockeries of a sham of reality.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_DISABLE_CURSORS\n" ,
   "------------------------------\n" ,
   "If this variable is set to YES, then AFNI will not try to change the X11\n" ,
   "cursor shape.  This feature is available because it seems that sometimes\n" ,
   "particular X11 installations choices of cursor and AFNI's choices don't\n" ,
   "work together well.  If you have unpleasant cursors in AFNI (e.g., an X),\n" ,
   "try setting this variable to YES.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_SLAVE_FUNCTIME (editable)\n" ,
   "-----------------------------\n" ,
   "When the underlay and overlay datasets both are time-dependent, switching\n" ,
   "the time index will change both the underlay and overlay sub-bricks.  If\n" ,
   "you want the time index control to change ONLY the underlay sub-brick,\n" ,
   "then set this variable to NO.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_SLAVE_THRTIME (editable)\n" ,
   "----------------------------\n" ,
   "When the underlay and overlay datasets both are time-dependent, switching\n" ,
   "the time index will change both the underlay and overlay sub-bricks, but\n" ,
   "NOT the threshold sub-brick.  If you want the time index control to change\n" ,
   "the threshold sub-brick, then set this variable to YES.\n" ,
   "\n" ,
   "----------------------------\n" ,
   "Variable: AFNI_CLICK_MESSAGE\n" ,
   "----------------------------\n" ,
   "If this veriable is set to NO, then the string\n" ,
   "  [---------------]\n" ,
   "  [ Click in Text ]\n" ,
   "  [ to Pop Down!! ]\n" ,
   "will NOT be appended to the very first popup message window that AFNI\n" ,
   "creates.  This message was added because some people do not realize\n" ,
   "that the way to get rid of these popups (before they vanish on their\n" ,
   "own after 30 seconds) is to click in them.  You know who you are.\n" ,
   "However, if you are advanced enough to read this file, then you probably\n" ,
   "aren't one of THEM.\n" ,
   "\n" ,
   "------------------------------------\n" ,
   "Variable: AFNI_REALTIME_MP_HOST_PORT (editable)\n" ,
   "------------------------------------\n" ,
   "When this variable is set, the realtime plugin will attempt to open a tcp\n" ,
   "socket to the corresponding host and port, and will send the six registration\n" ,
   "correction parameters for each 3D volume received by the plugin.  This applies\n" ,
   "only to the case of graphing 3D registration.  The socket will be opened at\n" ,
   "the start of each run, and will be closed at the end.  A simple example of\n" ,
   "what to set this variable to is localhost:53214.  See 'serial_helper -help'\n" ,
   "for more details.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_X11_REDECORATE (editable)\n" ,
   "-----------------------------\n" ,
   "By default, AFNI tries to change some of the \"decorations\" (control buttons)\n" ,
   "on some of the windows it creates (e.g., removing resize handles).  If you\n" ,
   "don't want this to happen, set this variable to NO.  This variable only\n" ,
   "has an effect on windows created AFTER it is set, so if you change this\n" ,
   "interactively in the Edit Environment plugin, it will not affect existing\n" ,
   "windows.  Normally, you would want to set this in your .afnirc file.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_IMAGE_SAVESQUARE\n" ,
   "-------------------------------\n" ,
   "YES/NO: Forces images (from the image view \"Save\" button) to be saved with\n" ,
   "square pixels, even if they are stored with nonsquare pixels.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_BUCKET_LABELSIZE\n" ,
   "-------------------------------\n" ,
   "THIS VARIABLE HAS BEEN REMOVED FROM AFNI.\n" ,
   "\n" ,
   "Formerly, it was used to set the width of the \"ULay\", \"OLay\", and \"Thr\" menu\n" ,
   "choosers on the \"Define Overlay\" control panel.  As of 03 May 2005, AFNI now\n" ,
   "calculates the default width based on the longest sub-brick label input\n" ,
   "for each dataset.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_MAX_1DSIZE\n" ,
   "-------------------------\n" ,
   "Sets the maximum size (in bytes) of each 1D file that will be automatically\n" ,
   "loaded when AFNI starts.  The default is 123 Kbytes.  The intention is to\n" ,
   "prevent loading of very large files that are not intended to be used for\n" ,
   "graphing/FIMming purposes.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_TITLE_LABEL2 (editable)\n" ,
   "---------------------------\n" ,
   "If this YES/NO variable is YES, then the AFNI window titlebars will show\n" ,
   "the 'label2' field from the AFNI dataset .HEAD file, rather than the\n" ,
   "dataset filename.  If the label2 field is set to a nontrivial value,\n" ,
   "that is.  You can set the label2 field with the 3drefit command.\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_SHOW_SURF_POPUPS\n" ,
   "-------------------------------\n" ,
   "If this YES/NO variable is set to YES, then when AFNI receives surface\n" ,
   "nodes, triangles or normals from suma, a popup message will be displayed.\n" ,
   "Otherwise, the message will be send to stderr (on the terminal window).\n" ,
   "\n" ,
   "-------------------------------\n" ,
   "Variable: AFNI_KILL_SURF_POPUPS\n" ,
   "-------------------------------\n" ,
   "If this YES/NO variable is set to YES, then when AFNI receives surface\n" ,
   "nodes, triangles or normals from suma, no messages will be displayed,\n" ,
   "either in a popup or stderr.  Note that if errors occur, popups will\n" ,
   "still be shown; this just turns off the normal information messages.\n" ,
   "N.B.: If AFNI_SHOW_SURF_POPUPS is YES, then it wins over\n" ,
   "      AFNI_KILL_SURF_POPUPS being YES.  If neither is set, then\n" ,
   "      messages are displayed to stderr.\n" ,
   "\n" ,
   "-----------------------------\n" ,
   "Variable: AFNI_EDGIZE_OVERLAY (editable)\n" ,
   "-----------------------------\n" ,
   "If set to YES, then the color overlays in the image windows will only\n" ,
   "have their edge pixels displayed.  That is, each 'blob' will be hollowed\n" ,
   "out, leaving only its edges.  If you do this, you probably want to make\n" ,
   "the color overlay opaque, so that the results are easily seen.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_NIFTI_DEBUG (editable)\n" ,
   "--------------------------\n" ,
   "This integral variable determines the debug level used by the nifti_io\n" ,
   "library functions.  If set to 0, only errors are reported by the library.\n" ,
   "The maximum debug level used is currently 4.  Note that if this is changed\n" ,
   "from within AFNI, a 'Rescan: This' operation should probably be performed,\n" ,
   "which will force a re-reading of the datasets and so force an elicitation\n" ,
   "of the NIfTI debug messages (for .nii files, that is).\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable AFNI_NIFTI_NOEXT\n" ,
   "-------------------------\n" ,
   "When writing a '.nii' (or '.nii.gz') file from an AFNI program, normally\n" ,
   "a NIfTI-1.1 extension field with some extra AFNI header information is\n" ,
   "written into the output file.  If you set this variable to YES, then\n" ,
   "this extension is not written, which will make the output be a 'pure'\n" ,
   "NIfTI-1.1 file.  Only use this if absolutely necessary.  You can also\n" ,
   "use the 'nifti_tool' program to strip extension data from a NIfTI-1.1\n" ,
   "dataset file.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_OVERLAY_ZERO (editable)\n" ,
   "---------------------------\n" ,
   "If set to YES, this variable indicates that voxels in the overlay dataset\n" ,
   "that have the numerical value of 0 will get colored when the Inten color\n" ,
   "scale on the Define Datamode panel indicates that 0 has a color that isn't\n" ,
   "\"none\".  The default way that AFNI works is NOT to colorize voxels that\n" ,
   "are 0, even if they should otherwise get a color.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: NIML_TRUSTHOST_xx\n" ,
   "---------------------------\n" ,
   "These environment variables ('xx' = '01', '02', ..., '99') set the names\n" ,
   "and/or addresses of external computer hosts to trust with NIML TCP/IP\n" ,
   "connections, which are how AFNI and SUMA communicate.  Should only be\n" ,
   "necessary to use these if you are using AFNI and SUMA on different\n" ,
   "machines.  Connections from machines not on the trusted list will be\n" ,
   "rejected, for the sake of security.  The 'localhost' or 127.0.0.1 address\n" ,
   "and local class B network 192.168.0.* addresses are always trusted.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_DONT_LOGFILE\n" ,
   "---------------------------\n" ,
   "Most AFNI programs write a copy of their command line to a file in your\n" ,
   "home directory named \".afni.log\".  If you do NOT want the log to be\n" ,
   "kept, set this environment variable to YES.  The purpose of the log\n" ,
   "is for you to be able to look back and see what AFNI commands you used\n" ,
   "in the past.  However, if you are doing a vast number of commands inside\n" ,
   "a script, the log file might eventually become gigantic (the Kevin Murphy\n" ,
   "effect).\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_WRITE_NIML\n" ,
   "-------------------------\n" ,
   "If this variable is set to YES, then AFNI .HEAD files will be written in\n" ,
   "the new NIML (XML subset) format, rather than the 'classic' format.  The\n" ,
   "volumetric image data is still in the pure binary .BRIK file, not XML-ified\n" ,
   "in any way.  At present (Jun 2005) this format is experimental, but will\n" ,
   "someday soon become the default.\n" ,
   "\n" ,
   "---------------------------------\n" ,
   "Variable: AFNI_ALLOW_MILLISECONDS\n" ,
   "---------------------------------\n" ,
   "The TR value (time step) in 3D+time datasets created with to3d can be flagged\n" ,
   "as being in units of milliseconds (ms) or seconds (s).  This situation is\n" ,
   "unfortunate, as some AFNI programs assume that the units are always s, which\n" ,
   "doesn't work well when the TR is actually in ms.  On 15 Aug 2005, AFNI dataset\n" ,
   "I/O was modified to only write out TR in s units, and to convert ms units\n" ,
   "to s units on input.  If you absolutely need to store TR in ms, then you\n" ,
   "must set this environment variable to YES.  I strongly recommend against\n" ,
   "such a setting, but recall the AFNI philosophy: \"provide mechanism, not\n" ,
   "policy\" -- in other words, if you want to shoot yourself in the foot, go\n" ,
   "right ahead.  This variable is just the safety on the revolver.\n" ,
   "\n" ,
   "------------------------------\n" ,
   "Variable: AFNI_TTATLAS_CAUTION (editable)\n" ,
   "------------------------------\n" ,
   "If this YES/NO variable is set to NO, then the warning about the potential\n" ,
   "errors in the \"Where am I?\" popup will not appear.  This is purely for\n" ,
   "cosmetic purposes, Ziad.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_AUTO_RESCAN\n" ,
   "--------------------------\n" ,
   "If this YES/NO variable is set to YES, then the interactive AFNI program\n" ,
   "will rescan all session directories every 15 seconds for new datasets.\n" ,
   "Basically, this is just a way for you to avoid pressing the 'Rescan'\n" ,
   "buttons.  Note that if AFNI_AUTO_RESCAN is enabled, then the rescan\n" ,
   "method will be 'Add', not 'Replace', no matter what you set variable\n" ,
   "AFNI_RESCAN_METHOD to.\n" ,
   "\n" ,
   "=============================================\n" ,
   "| Robert W Cox, PhD                         |\n" ,
   "| Scientific and Statistical Computing Core |\n" ,
   "| National Institute of Mental Health       |\n" ,
   "| National Institutes of Health             |\n" ,
   "| Department of Health & Human Services     |\n" ,
   "| United States of America                  |\n" ,
   "| Earth, United Federation of Planets       |\n" ,
   "=============================================\n" ,
   "\n" ,
   "############################################################################\n" ,
   "##### Variables that Specifically Affect the Operation of 3dDeconvolve #####\n" ,
   "############################################################################\n" ,
   "\n" ,
   "----------------------------------\n" ,
   "Variable: AFNI_3dDeconvolve_extend\n" ,
   "----------------------------------\n" ,
   "If you input a stimulus time series (via the -stim_file option) to\n" ,
   "3dDeconvolve that is shorter than needed for the regression analysis, the\n" ,
   "program will normally print a warning message and extend the time series\n" ,
   "with zero values to the needed length.  If you would rather have the program\n" ,
   "stop if it detects this problem (the behavior before 22 Oct 2003), then set\n" ,
   "this environment variable to NO.\n" ,
   "\n" ,
   "---------------------------------\n" ,
   "Variable: AFNI_3dDeconvolve_nodup\n" ,
   "---------------------------------\n" ,
   "If this variable is set to YES, then if the 3dDeconvolve program detects\n" ,
   "duplicate input stimulus filenames or duplicate regressors, the program\n" ,
   "will fail (with an error message) rather than attempt to continue.\n" ,
   "\n" ,
   "-----------------------------------------\n" ,
   "Variable: AFNI_3dDeconvolve_nodata_extras\n" ,
   "-----------------------------------------\n" ,
   "When using the -nodata option in 3dDeconvolve, the default printout gives\n" ,
   "the 'normalized standard deviation' for each stimulus parameter.  If you\n" ,
   "set this variable to YES, then the printout will include the -polort\n" ,
   "baseline parameters as well, and also the L2 norm of each column in\n" ,
   "the regression matrix.\n" ,
   "\n" ,
   "--------------------------\n" ,
   "Variable: AFNI_XJPEG_COLOR\n" ,
   "--------------------------\n" ,
   "Determines the color of the lines drawn between the column boxes in the\n" ,
   "output from the -xjpeg option to 3dDeconvolve.  The color format is\n" ,
   "\"rgbi:rf/gf/bf\", where each value rf,gf,bf is a number between 0.0 and 1.0\n" ,
   "(inclusive); for example, yellow would be \"rgbi:1.0/1.0/0.0\".  As a\n" ,
   "special case, if this value is the string \"none\" or \"NONE\", then these\n" ,
   "lines will not be drawn.\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_XJPEG_IMXY\n" ,
   "-------------------------\n" ,
   "This variable determines the size of the image saved when via the -xjpeg\n" ,
   "option to 3dDeconvolve.  It should be in the format AxB, where 'A' is the\n" ,
   "number of pixels the image is to be wide (across the matrix rows) and 'B'\n" ,
   "is the number of pixels high (down the columns); for example:\n" ,
   "  setenv AFNI_XJPEG_IMXY 768x1024\n" ,
   "which means to set the x-size (horizontal) to 768 pixels and the y-size\n" ,
   "(vertical) to 1024 pixels.  These values are the default, by the way.\n" ,
   "\n" ,
   "If the first value 'A' is negative and less than -1, its absolute value\n" ,
   "is the number of pixels across PER ROW.  If the second value 'B' is\n" ,
   "negative, its absolute value is the number of pixels down PER ROW.\n" ,
   "(Usually there are many fewer columns than rows.)\n" ,
   "\n" ,
   "-------------------------\n" ,
   "Variable: AFNI_XSAVE_TEXT\n" ,
   "-------------------------\n" ,
   "If this YES/NO variable is set to YES, then the .xsave file created by\n" ,
   "the \"-xsave\" option to 3dDeconvolve will be saved in text format.  The\n" ,
   "default is a binary format, which preserves the full accuracy of the\n" ,
   "matrices stored therein.  However, if you want to look at the .xsave\n" ,
   "file yourself, the binary format is hard to grok.  Note that the two\n" ,
   "forms are not quite equivalent, since the binary format stores the\n" ,
   "exact matrices used internally in the program, whereas the ASCII format\n" ,
   "stores only a decimal approximation of these matrices.\n" ,
   "\n" ,
   "---------------------------\n" ,
   "Variable: AFNI_GLTSYM_PRINT\n" ,
   "---------------------------\n" ,
   "If this YES/NO variable is set to YES, then the GLT matrices generated\n" ,
   "in 3dDeconvolve by the \"-gltsym\" option will be printed to the screen\n" ,
   "when the program starts up.\n" ,
   NULL } ;
#define NUM_readme_env 2281
